@@ -22,9 +22,11 @@
 
 import urllib, os, hashlib
 from urllib.parse import urlparse
-from http.client import HTTPConnection, _CS_IDLE
+from http.client import HTTPConnection, _CS_IDLE, HTTPException
 from os import listdir
 from os.path import isfile, join
+import re
+import argparse
 
 #################################################################################################################################
 class HTTPGetCallback:
@@ -54,7 +56,7 @@ class HTTPGetter:
 
         try:
             self.doGet(httpGetCallback)
-        except:
+        except HTTPException:
             # try to reconnect once
             #print("reconnect")
             self.httpConnection.close()
@@ -117,6 +119,9 @@ class HTTPDownloadRequest(HTTPGetCallback):
         self.mycallback = callback
 
     def callback(self):
+        if self.result.status != 200:
+            return
+
         with open(self.dst, 'wb') as f:
             f.write(self.result.read())
 
@@ -140,13 +145,55 @@ def hash_of_file(fname):
     return hash.hexdigest()
 
 #################################################################################################################################
+
+class Coordinate:
+    def __init__(self, lat, lon):
+        self.lat = lat
+        self.lon = lon
+
+class DownloadBoundaries:
+    def __init__(self, top, left, bottom, right):
+        if top < bottom:
+            raise ValueError("top cannot be less than bottom")
+        if right < left:
+            raise ValueError("right cannot be less than left")
+
+        if top > 90 or bottom < -90:
+            raise ValueError("top and bottom must be a valid latitude")
+        if left < -180 or right > 180:
+            raise ValueError("left and right must be a valid longitude")
+        self.top = top
+        self.left = left
+        self.bottom = bottom
+        self.right = right
+
+    def is_coordinate_inside_boundaries(self, coordinate):
+        if coordinate.lat < self.bottom or coordinate.lat > self.top:
+            return False
+        if coordinate.lon < self.left or coordinate.lon > self.right:
+            return False
+        return True
+
+def parse_terrasync_coordinate(coordinate):
+    matches = re.match("(w|e)(\d{3})(n|s)(\d{2})", coordinate)
+    if not matches:
+        return None
+    lon = int(matches.group(2))
+    if matches.group(1) == "w":
+        lon *= -1
+    lat = int(matches.group(4))
+    if matches.group(3) == "s":
+        lat *= -1
+    return Coordinate(lat, lon)
+
 class TerraSync:
 
-    def __init__(self, url="http://flightgear.sourceforge.net/scenery", target=".", quick=False, removeOrphan=False):
+    def __init__(self, url, target, quick, removeOrphan, downloadBoundaries):
         self.setUrl(url).setTarget(target)
         self.quick = quick
         self.removeOrphan = removeOrphan
         self.httpGetter = None
+        self.downloadBoundaries = downloadBoundaries
 
     def setUrl(self, url):
         self.url = url.rstrip('/').strip()
@@ -175,6 +222,12 @@ class TerraSync:
     def updateDirectory(self, serverPath, localPath, dirIndexHash):
         print("processing ", serverPath)
 
+        if len(serverPath) > 0:
+            serverFolderName = serverPath[serverPath.rfind('/') + 1:]
+            coordinate = parse_terrasync_coordinate(serverFolderName)
+            if coordinate and not self.downloadBoundaries.is_coordinate_inside_boundaries(coordinate):
+                return
+
         localFullPath = join(self.target, localPath)
         if not os.path.exists( localFullPath ):
           os.makedirs( localFullPath )
@@ -224,26 +277,30 @@ class TerraSync:
             self.httpGetter.update()
 
 #################################################################################################################################
-import getopt, sys
-try:
-  opts, args = getopt.getopt(sys.argv[1:], "u:t:qr", [ "url=", "target=", "quick", "remove-orphan" ])
 
-except getopt.GetoptError:
-  print("terrasync.py [--url=http://some.server.org/scenery] [--target=/some/path] [-q|--quick] [-r|--remove-orphan]")
-  sys.exit(2)
 
-terraSync = TerraSync()
-for opt, arg in opts:
-  if opt in("-u", "--url"):
-    terraSync.url = arg
+parser = argparse.ArgumentParser()
+parser.add_argument("-u", "--url", dest="url", metavar="URL",
+        default="http://flightgear.sourceforge.net/scenery", help="Server URL [default: %(default)s]")
+parser.add_argument("-t", "--target", dest="target", metavar="DIR",
+        default=".", help="Directory to store the files [default: current directory]")
+parser.add_argument("-q", "--quick", dest="quick", action="store_true",
+        default=False, help="Quick")
+parser.add_argument("-r", "--remove-orphan", dest="removeOrphan", action="store_true",
+        default=False, help="Remove old scenery files")
 
-  elif opt in ("-t", "--target"):
-    terraSync.target = arg
+parser.add_argument("--top", dest="top", type=int,
+        default=90, help="Maximum latitude to include in download [default: %(default)d]")
+parser.add_argument("--bottom", dest="bottom", type=int,
+        default=-90, help="Minimum latitude to include in download [default: %(default)d]")
+parser.add_argument("--left", dest="left", type=int,
+        default=-180, help="Minimum longitude to include in download [default: %(default)d]")
+parser.add_argument("--right", dest="right", type=int,
+        default=180, help="Maximum longitude to include in download [default: %(default)d]")
 
-  elif opt in ("-q", "--quick"):
-    terraSync.quick = True
+args = parser.parse_args()
 
-  elif opt in ("-r", "--remove-orphan"):
-    terraSync.removeOrphan = True
+terraSync = TerraSync(args.url, args.target, args.quick, args.removeOrphan,
+        DownloadBoundaries(args.top, args.left, args.bottom, args.right))
 
 terraSync.start()
