@@ -17,7 +17,6 @@
 namespace yasim {
 
 // gadgets
-inline float norm(float f) { return f<1 ? 1/f : f; }
 inline float abs(float f) { return f<0 ? -f : f; }
 
 // Solver threshold.  How close to the solution are we trying
@@ -34,29 +33,7 @@ const float SOLVE_TWEAK = 0.3226;
 
 Airplane::Airplane()
 {
-    _emptyWeight = 0;
-    _pilotPos[0] = _pilotPos[1] = _pilotPos[2] = 0;
-    _wing = 0;
-    _tail = 0;
-    _ballast = 0;
-    _cruiseP = 0;
-    _cruiseT = 0;
-    _cruiseSpeed = 0;
-    _cruiseWeight = 0;
-    _cruiseGlideAngle = 0;
-    _approachP = 0;
-    _approachT = 0;
-    _approachSpeed = 0;
-    _approachAoA = 0;
-    _approachWeight = 0;
-    _approachGlideAngle = 0;
-
-    _dragFactor = 1;
-    _liftRatio = 1;
-    _cruiseAoA = 0;
-    _tailIncidence = 0;
-
-    _failureMsg = 0;
+    _approachConfig.isApproach = true;
 }
 
 Airplane::~Airplane()
@@ -82,10 +59,10 @@ Airplane::~Airplane()
     }
     for(i=0; i<_solveWeights.size(); i++)
         delete (SolveWeight*)_solveWeights.get(i);
-    for(i=0; i<_cruiseControls.size(); i++)
-        delete (Control*)_cruiseControls.get(i);
-    for(i=0; i<_approachControls.size(); i++) {
-        Control* c = (Control*)_approachControls.get(i);
+    for(i=0; i<_cruiseConfig.controls.size(); i++)
+        delete (Control*)_cruiseConfig.controls.get(i);
+    for(i=0; i<_approachConfig.controls.size(); i++) {
+        Control* c = (Control*)_approachConfig.controls.get(i);
         if(c != &_approachElevator)
             delete c;
     }
@@ -113,17 +90,7 @@ void Airplane::calcFuelWeights()
     }
 }
 
-ControlMap* Airplane::getControlMap()
-{
-    return &_controls;
-}
-
-Model* Airplane::getModel()
-{
-    return &_model;
-}
-
-void Airplane::getPilotAccel(float* out)
+void Airplane::getPilotAccel(float* out) 
 {
     State* s = _model.getState();
 
@@ -133,55 +100,17 @@ void Airplane::getPilotAccel(float* out)
     Math::vmul33(s->orient, out, out);
     out[0] = -out[0];
 
-    // The regular acceleration
-    float tmp[3];
+    float acceleration[3];
     // Convert to aircraft coordinates
-    Math::vmul33(s->orient, s->acc, tmp);
-    tmp[1] = -tmp[1];
-    tmp[2] = -tmp[2];
+    Math::vmul33(s->orient, s->acc, acceleration);
+    acceleration[1] = -acceleration[1];
+    acceleration[2] = -acceleration[2];
 
-    Math::add3(tmp, out, out);
+    Math::add3(acceleration, out, out);
 
     // FIXME: rotational & centripetal acceleration needed
 }
 
-void Airplane::setPilotPos(float* pos)
-{
-    int i;
-    for(i=0; i<3; i++) _pilotPos[i] = pos[i];
-}
-
-void Airplane::getPilotPos(float* out)
-{
-    int i;
-    for(i=0; i<3; i++) out[i] = _pilotPos[i];
-}
-
-int Airplane::numGear()
-{
-    return _gears.size();
-}
-
-Gear* Airplane::getGear(int g)
-{
-    return ((GearRec*)_gears.get(g))->gear;
-}
-
-Hook* Airplane::getHook()
-{
-    return _model.getHook();
-}
-
-Launchbar* Airplane::getLaunchbar()
-{
-    return _model.getLaunchbar();
-}
-
-Rotorgear* Airplane::getRotorgear()
-{
-    return _model.getRotorgear();
-}
-
 void Airplane::updateGearState()
 {
     for(int i=0; i<_gears.size(); i++) {
@@ -196,30 +125,29 @@ void Airplane::updateGearState()
 
 void Airplane::setApproach(float speed, float altitude, float aoa, float fuel, float gla)
 {
-    _approachSpeed = speed;
-    _approachP = Atmosphere::getStdPressure(altitude);
-    _approachT = Atmosphere::getStdTemperature(altitude);
-    _approachAoA = aoa;
-    _approachFuel = fuel;
-    _approachGlideAngle = gla;
+    _approachConfig.speed = speed;
+    _approachConfig.altitude = altitude;
+    _approachConfig.state.setupOrientationFromAoa(aoa); // see runConfig()
+    _approachConfig.aoa = aoa; // not strictly needed, see runConfig()
+    _approachConfig.fuel = fuel;
+    _approachConfig.glideAngle = gla;
 }
  
 void Airplane::setCruise(float speed, float altitude, float fuel, float gla)
 {
-    _cruiseSpeed = speed;
-    _cruiseP = Atmosphere::getStdPressure(altitude);
-    _cruiseT = Atmosphere::getStdTemperature(altitude);
-    _cruiseAoA = 0;
+    _cruiseConfig.speed = speed;
+    _cruiseConfig.altitude = altitude;
+    _cruiseConfig.aoa = 0;
     _tailIncidence = 0;
-    _cruiseFuel = fuel;
-    _cruiseGlideAngle = gla;
+    _cruiseConfig.fuel = fuel;
+    _cruiseConfig.glideAngle = gla;
 }
 
 void Airplane::setElevatorControl(int control)
 {
     _approachElevator.control = control;
     _approachElevator.val = 0;
-    _approachControls.add(&_approachElevator);
+    _approachConfig.controls.add(&_approachElevator);
 }
 
 void Airplane::addApproachControl(int control, float val)
@@ -227,7 +155,7 @@ void Airplane::addApproachControl(int control, float val)
     Control* c = new Control();
     c->control = control;
     c->val = val;
-    _approachControls.add(c);
+    _approachConfig.controls.add(c);
 }
 
 void Airplane::addCruiseControl(int control, float val)
@@ -235,7 +163,7 @@ void Airplane::addCruiseControl(int control, float val)
     Control* c = new Control();
     c->control = control;
     c->val = val;
-    _cruiseControls.add(c);
+    _cruiseConfig.controls.add(c);
 }
 
 void Airplane::addSolutionWeight(bool approach, int idx, float wgt)
@@ -247,51 +175,6 @@ void Airplane::addSolutionWeight(bool approach, int idx, float wgt)
     _solveWeights.add(w);
 }
 
-int Airplane::numTanks()
-{
-    return _tanks.size();
-}
-
-float Airplane::getFuel(int tank)
-{
-    return ((Tank*)_tanks.get(tank))->fill;
-}
-
-float Airplane::setFuel(int tank, float fuel)
-{
-    return ((Tank*)_tanks.get(tank))->fill = fuel;
-}
-
-float Airplane::getFuelDensity(int tank)
-{
-    return ((Tank*)_tanks.get(tank))->density;
-}
-
-float Airplane::getTankCapacity(int tank)
-{
-    return ((Tank*)_tanks.get(tank))->cap;
-}
-
-void Airplane::setWeight(float weight)
-{
-    _emptyWeight = weight;
-}
-
-void Airplane::setWing(Wing* wing)
-{
-    _wing = wing;
-}
-
-void Airplane::setTail(Wing* tail)
-{
-    _tail = tail;
-}
-
-void Airplane::addVStab(Wing* vstab)
-{
-    _vstabs.add(vstab);
-}
-
 void Airplane::addFuselage(float* front, float* back, float width,
                            float taper, float mid, 
                            float cx, float cy, float cz, float idrag)
@@ -332,21 +215,6 @@ void Airplane::addGear(Gear* gear)
     _gears.add(g);
 }
 
-void Airplane::addHook(Hook* hook)
-{
-    _model.addHook(hook);
-}
-
-void Airplane::addHitch(Hitch* hitch)
-{
-    _model.addHitch(hitch);
-}
-
-void Airplane::addLaunchbar(Launchbar* launchbar)
-{
-    _model.addLaunchbar(launchbar);
-}
-
 void Airplane::addThruster(Thruster* thruster, float mass, float* cg)
 {
     ThrustRec* t = new ThrustRec();
@@ -357,12 +225,14 @@ void Airplane::addThruster(Thruster* thruster, float mass, float* cg)
     _thrusters.add(t);
 }
 
+/// Use ballast to redistribute mass, this is NOT added to empty weight.
 void Airplane::addBallast(float* pos, float mass)
 {
-    _model.getBody()->addMass(mass, pos);
+    _model.getBody()->addMass(mass, pos, true);
     _ballast += mass;
 }
 
+/// Masses configurable at runtime, e.g. cargo, pax
 int Airplane::addWeight(float* pos, float size)
 {
     WeightRec* wr = new WeightRec();
@@ -377,6 +247,7 @@ int Airplane::addWeight(float* pos, float size)
     return _weights.add(wr);
 }
 
+/// Change weight of a previously added mass point
 void Airplane::setWeight(int handle, float mass)
 {
     WeightRec* wr = (WeightRec*)_weights.get(handle);
@@ -398,96 +269,68 @@ void Airplane::setWeight(int handle, float mass)
 
 void Airplane::setFuelFraction(float frac)
 {
-    int i;
-    for(i=0; i<_tanks.size(); i++) {
+    for(int i=0; i<_tanks.size(); i++) {
         Tank* t = (Tank*)_tanks.get(i);
         t->fill = frac * t->cap;
         _model.getBody()->setMass(t->handle, t->cap * frac);
     }
 }
 
-float Airplane::getDragCoefficient()
-{
-    return _dragFactor;
-}
-
-float Airplane::getLiftRatio()
-{
-    return _liftRatio;
-}
-
-float Airplane::getCruiseAoA()
-{
-    return _cruiseAoA;
-}
-
-float Airplane::getTailIncidence()
-{
-    return _tailIncidence;
-}
-
-const char* Airplane::getFailureMsg()
-{
-    return _failureMsg;
-}
-
-int Airplane::getSolutionIterations()
-{
-    return _solutionIterations;
-}
-
-void Airplane::setupState(float aoa, float speed, float gla, State* s)
-{
-    float cosAoA = Math::cos(aoa);
-    float sinAoA = Math::sin(aoa);
-    s->orient[0] =  cosAoA; s->orient[1] = 0; s->orient[2] = sinAoA;
-    s->orient[3] =       0; s->orient[4] = 1; s->orient[5] =      0;
-    s->orient[6] = -sinAoA; s->orient[7] = 0; s->orient[8] = cosAoA;
-
-    s->v[0] = speed*Math::cos(gla); s->v[1] = -speed*Math::sin(gla); s->v[2] = 0;
-
-    int i;
-    for(i=0; i<3; i++)
-	s->pos[i] = s->rot[i] = s->acc[i] = s->racc[i] = 0;
-
-    // Put us 1m above the origin, or else the gravity computation in
-    // Model goes nuts
-    s->pos[2] = 1;
-}
+/**
+ * @brief add contact point for crash detection 
+ * used to add wingtips and fuselage nose and tail
+ * 
+ * @param pos ...
+ */
 
 void Airplane::addContactPoint(float* pos)
 {
     ContactRec* c = new ContactRec;
     c->gear = 0;
-    c->p[0] = pos[0];
-    c->p[1] = pos[1];
-    c->p[2] = pos[2];
+    Math::set3(pos, c->p);
     _contacts.add(c);
 }
 
 float Airplane::compileWing(Wing* w)
 {
-    // The tip of the wing is a contact point
-    float tip[3];
-    w->getTip(tip);
-    addContactPoint(tip);
-    if(w->isMirrored()) {
-        tip[1] *= -1;
-        addContactPoint(tip);
-    }
-
     // Make sure it's initialized.  The surfaces will pop out with
     // total drag coefficients equal to their areas, which is what we
     // want.
     w->compile();
 
-    float wgt = 0;
-    int i;
-    for(i=0; i<w->numSurfaces(); i++) {
-        Surface* s = (Surface*)w->getSurface(i);
+    // The tip of the wing is a contact point
+    float tip[3];
+    // need compile() before getTip()!
+    w->getTip(tip);
+    addContactPoint(tip);
+    if(w->isMirrored()) {
+        tip[1] *= -1;
+        addContactPoint(tip);
+        tip[1] *= -1; //undo mirror
+    }
+    if (_wingsN != 0) {
+      _wingsN->getNode("tip-x", true)->setFloatValue(tip[0]);
+      _wingsN->getNode("tip-y", true)->setFloatValue(tip[1]);
+      _wingsN->getNode("tip-z", true)->setFloatValue(tip[2]);
+      w->getBase(tip);
+      _wingsN->getNode("base-x", true)->setFloatValue(tip[0]);
+      _wingsN->getNode("base-y", true)->setFloatValue(tip[1]);
+      _wingsN->getNode("base-z", true)->setFloatValue(tip[2]);
+      _wingsN->getNode("wing-span", true)->setFloatValue(w->getSpan());
+      _wingsN->getNode("wing-area", true)->setFloatValue(w->getArea());
+      _wingsN->getNode("aspect-ratio", true)->setFloatValue(w->getAspectRatio());
+      _wingsN->getNode("standard-mean-chord", true)->setFloatValue(w->getSMC());
+      _wingsN->getNode("mac", true)->setFloatValue(w->getMAC());
+      _wingsN->getNode("mac-x", true)->setFloatValue(w->getMACx());
+      _wingsN->getNode("mac-y", true)->setFloatValue(w->getMACy());
+    }
 
-	float td = s->getTotalDrag();
-	s->setTotalDrag(td);
+    float wgt = 0;
+    float dragSum = 0;
+    for(int i=0; i<w->numSurfaces(); i++) {
+        Surface* s = (Surface*)w->getSurface(i);
+        float td = s->getTotalDrag();
+        int sid = s->getID();
 
         _model.addSurface(s);
 
@@ -495,8 +338,18 @@ float Airplane::compileWing(Wing* w)
         mass = mass * Math::sqrt(mass);
         float pos[3];
         s->getPosition(pos);
-        _model.getBody()->addMass(mass, pos);
+        int mid = _model.getBody()->addMass(mass, pos, true);
+        if (_wingsN != 0) {
+          SGPropertyNode_ptr n = _wingsN->getNode("surfaces", true)->getChild("surface", sid, true);
+          n->getNode("drag", true)->setFloatValue(td);
+          n->getNode("mass-id", true)->setIntValue(mid);
+        }
         wgt += mass;
+        dragSum += td;
+    }
+    if (_wingsN != 0)  {
+      _wingsN->getNode("weight", true)->setFloatValue(wgt);	
+      _wingsN->getNode("drag", true)->setFloatValue(dragSum);	
     }
     return wgt;
 }
@@ -547,7 +400,7 @@ float Airplane::compileFuselage(Fuselage* f)
 
         // _Mass_ weighting goes as surface area^(3/2)
         float mass = scale*segWgt * Math::sqrt(scale*segWgt);
-        _model.getBody()->addMass(mass, pos);
+        _model.getBody()->addMass(mass, pos, true);
         wgt += mass;
 
         // Make a Surface too
@@ -635,6 +488,13 @@ void Airplane::compileGear(GearRec* gr)
     _surfs.add(s);
 }
 
+/**
+ * @brief add "fake gear" per contact point
+ * 
+ * 
+ * @return void
+ */
+
 void Airplane::compileContactPoints()
 {
     // Figure it will compress by 20cm
@@ -648,8 +508,7 @@ void Airplane::compileContactPoints()
     float spring = (1/DIST) * 9.8f * 10.0f * mass;
     float damp = 2 * Math::sqrt(spring * mass);
 
-    int i;
-    for(i=0; i<_contacts.size(); i++) {
+    for(int i=0; i<_contacts.size(); i++) {
         ContactRec* c = (ContactRec*)_contacts.get(i);
 
         Gear* g = new Gear();
@@ -674,6 +533,7 @@ void Airplane::compile()
 {
     RigidBody* body = _model.getBody();
     int firstMass = body->numMasses();
+    SGPropertyNode_ptr baseN = fgGetNode("/fdm/yasim/model/wings", true);
 
     // Generate the point masses for the plane.  Just use unitless
     // numbers for a first pass, then go back through and rescale to
@@ -682,13 +542,30 @@ void Airplane::compile()
 
     // The Wing objects
     if (_wing)
+    {
+      if (baseN != 0) _wingsN = baseN->getChild("wing", 0, true);
       aeroWgt += compileWing(_wing);
+      
+      // convert % to absolute x coordinates
+      _cgDesiredFront = _wing->getMACx() - _wing->getMAC()*_cgDesiredMin;
+      _cgDesiredAft = _wing->getMACx() - _wing->getMAC()*_cgDesiredMax;
+      if (baseN != 0) {
+        SGPropertyNode_ptr n = fgGetNode("/fdm/yasim/model", true);
+        n->getNode("cg-x-range-front", true)->setFloatValue(_cgDesiredFront);
+        n->getNode("cg-x-range-aft", true)->setFloatValue(_cgDesiredAft);
+      }
+    }
     if (_tail)
+    {
+      if (baseN != 0) _wingsN = baseN->getChild("tail", 0, true);
       aeroWgt += compileWing(_tail);
+    }
     int i;
     for(i=0; i<_vstabs.size(); i++)
-        aeroWgt += compileWing((Wing*)_vstabs.get(i)); 
-
+    {
+      if (baseN != 0) _wingsN = baseN->getChild("stab", i, true);
+      aeroWgt += compileWing((Wing*)_vstabs.get(i));
+    }
 
     // The fuselage(s)
     for(i=0; i<_fuselages.size(); i++)
@@ -707,7 +584,7 @@ void Airplane::compile()
     // Add the thruster masses
     for(i=0; i<_thrusters.size(); i++) {
         ThrustRec* t = (ThrustRec*)_thrusters.get(i);
-        body->addMass(t->mass, t->cg);
+        body->addMass(t->mass, t->cg, true);
     }
 
     // Add the tanks, empty for now.
@@ -717,9 +594,10 @@ void Airplane::compile()
         t->handle = body->addMass(0, t->pos);
         totalFuel += t->cap;
     }
-    _cruiseWeight = _emptyWeight + totalFuel*_cruiseFuel;
-    _approachWeight = _emptyWeight + totalFuel*_approachFuel;
+    _cruiseConfig.weight = _emptyWeight + totalFuel*_cruiseConfig.fuel;
+    _approachConfig.weight = _emptyWeight + totalFuel*_approachConfig.fuel;
 
+    
     body->recalc();
 
     // Add surfaces for the landing gear.
@@ -731,20 +609,35 @@ void Airplane::compile()
         ThrustRec* tr = (ThrustRec*)_thrusters.get(i);
         tr->handle = _model.addThruster(tr->thruster);
     }
-
-    // Ground effect
+    
     if(_wing) {
-        float gepos[3];
+        // Ground effect
+        // If a double tapered wing is modelled with wing and mstab, wing must 
+        // be outboard to get correct wingspan.
+        float pos[3];
         float gespan = 0;
-        gespan = _wing->getGroundEffect(gepos);
-        _model.setGroundEffect(gepos, gespan, 0.15f);
+        gespan = _wing->getSpan();
+        _wing->getBase(pos);
+        if(!isVersionOrNewer( Version::YASIM_VERSION_2017_2 )) {
+          //old code
+          //float span = _length * Math::cos(_sweep) * Math::cos(_dihedral);
+          //span = 2*(span + Math::abs(_base[2]));
+          gespan -= 2*pos[1]; // cut away base (y-distance)
+          gespan += 2*Math::abs(pos[2]); // add (wrong) z-distance
+        }
+        if (baseN != 0)
+          baseN->getChild("wing", 0)->getNode("gnd-eff-span", true)->setFloatValue(gespan);
+        // where does the hard coded factor 0.15 come from?
+        _model.setGroundEffect(pos, gespan, 0.15f);
     }
-
+    
     // solve function below resets failure message
     // so check if we have any problems and abort here
     if (_failureMsg) return;
 
     solveGear();
+    calculateCGHardLimits();
+    
     if(_wing && _tail) solve();
     else
     {
@@ -787,12 +680,12 @@ void Airplane::solveGear()
     // The force at max compression should be sufficient to stop a
     // plane moving downwards at 2x the approach descent rate.  Assume
     // a 3 degree approach.
-    float descentRate = 2.0f*_approachSpeed/19.1f;
+    float descentRate = 2.0f*_approachConfig.speed/19.1f;
 
     // Spread the kinetic energy according to the gear weights.  This
     // will results in an equal compression fraction (not distance) of
     // each gear.
-    float energy = 0.5f*_approachWeight*descentRate*descentRate;
+    float energy = 0.5f*_approachConfig.weight*descentRate*descentRate;
 
     for(i=0; i<_gears.size(); i++) {
         GearRec* gr = (GearRec*)_gears.get(i);
@@ -807,26 +700,39 @@ void Airplane::solveGear()
         gr->gear->setSpring(k * gr->gear->getSpring());
 
         // Critically damped (too damped, too!)
-        gr->gear->setDamping(2*Math::sqrt(k*_approachWeight*gr->wgt)
+        gr->gear->setDamping(2*Math::sqrt(k*_approachConfig.weight*gr->wgt)
                              * gr->gear->getDamping());
     }
 }
 
+void Airplane::calculateCGHardLimits()
+{
+    _cgMax = -1e6;
+    _cgMin = 1e6;
+    for (int i = 0; i < _gears.size(); i++) {
+        GearRec* gr = (GearRec*)_gears.get(i);
+        float pos[3];
+        gr->gear->getPosition(pos);
+        if (pos[0] > _cgMax) _cgMax = pos[0];
+        if (pos[0] < _cgMin) _cgMin = pos[0];
+    }
+}
+
 void Airplane::initEngines()
 {
     for(int i=0; i<_thrusters.size(); i++) {
         ThrustRec* tr = (ThrustRec*)_thrusters.get(i);
-	tr->thruster->init();
+        tr->thruster->init();
     }
 }
 
 void Airplane::stabilizeThrust()
 {
-    int i;
-    for(i=0; i<_thrusters.size(); i++)
+    for(int i=0; i<_thrusters.size(); i++)
 	_model.getThruster(i)->stabilize();
 }
 
+/// Setup weights for cruise or approach during solve.
 void Airplane::setupWeights(bool isApproach)
 {
     int i;
@@ -839,93 +745,56 @@ void Airplane::setupWeights(bool isApproach)
     }
 }
 
-void Airplane::runCruise()
+/// load values for controls as defined in cruise configuration
+void Airplane::loadControls(const Vector& controls)
 {
-    setupState(_cruiseAoA, _cruiseSpeed,_cruiseGlideAngle, &_cruiseState);
-    _model.setState(&_cruiseState);
-    _model.setAir(_cruiseP, _cruiseT,
-                  Atmosphere::calcStdDensity(_cruiseP, _cruiseT));
-
-    // The control configuration
-    _controls.reset();
-    int i;
-    for(i=0; i<_cruiseControls.size(); i++) {
-	Control* c = (Control*)_cruiseControls.get(i);
-	_controls.setInput(c->control, c->val);
-    }
-    _controls.applyControls(1000000); // Huge dt value
-
-    // The local wind
-    float wind[3];
-    Math::mul3(-1, _cruiseState.v, wind);
-    Math::vmul33(_cruiseState.orient, wind, wind);
- 
-    setFuelFraction(_cruiseFuel);
-    setupWeights(false);
-   
-    // Set up the thruster parameters and iterate until the thrust
-    // stabilizes.
-    for(i=0; i<_thrusters.size(); i++) {
-	Thruster* t = ((ThrustRec*)_thrusters.get(i))->thruster;
-	t->setWind(wind);
-	t->setAir(_cruiseP, _cruiseT,
-                  Atmosphere::calcStdDensity(_cruiseP, _cruiseT));
-    }
-    stabilizeThrust();
-
-    updateGearState();
-
-    // Precompute thrust in the model, and calculate aerodynamic forces
-    _model.getBody()->recalc();
-    _model.getBody()->reset();
-    _model.initIteration();
-    _model.calcForces(&_cruiseState);
+  _controls.reset();
+  for(int i=0; i < controls.size(); i++) {
+    Control* c = (Control*)controls.get(i);
+    _controls.setInput(c->control, c->val);
+  }
+  _controls.applyControls(); 
 }
 
-void Airplane::runApproach()
+/// Helper for solve()
+void Airplane::runConfig(Config &cfg)
 {
-    setupState(_approachAoA, _approachSpeed,_approachGlideAngle, &_approachState);
-    _model.setState(&_approachState);
-    _model.setAir(_approachP, _approachT,
-                  Atmosphere::calcStdDensity(_approachP, _approachT));
-
-    // The control configuration
-    _controls.reset();
-    int i;
-    for(i=0; i<_approachControls.size(); i++) {
-	Control* c = (Control*)_approachControls.get(i);
-	_controls.setInput(c->control, c->val);
-    }
-    _controls.applyControls(1000000);
-
-    // The local wind
-    float wind[3];
-    Math::mul3(-1, _approachState.v, wind);
-    Math::vmul33(_approachState.orient, wind, wind);
-    
-    setFuelFraction(_approachFuel);
-
-    setupWeights(true);
-
-    // Run the thrusters until they get to a stable setting.  FIXME:
-    // this is lots of wasted work.
-    for(i=0; i<_thrusters.size(); i++) {
-	Thruster* t = ((ThrustRec*)_thrusters.get(i))->thruster;
-	t->setWind(wind);
-	t->setAir(_approachP, _approachT,
-                  Atmosphere::calcStdDensity(_approachP, _approachT));
-    }
-    stabilizeThrust();
-
-    updateGearState();
-
-    // Precompute thrust in the model, and calculate aerodynamic forces
-    _model.getBody()->recalc();
-    _model.getBody()->reset();
-    _model.initIteration();
-    _model.calcForces(&_approachState);
+  // aoa is consider to be given for approach so we calculate orientation 
+  // only once in setApproach()
+  if (!cfg.isApproach) {
+    cfg.state.setupOrientationFromAoa(cfg.aoa);
+  }
+  cfg.state.setupSpeedAndPosition(cfg.speed, cfg.glideAngle);
+  _model.setState(&cfg.state);
+  _model.setStandardAtmosphere(cfg.altitude);
+  loadControls(cfg.controls);
+  
+  // The local wind
+  float wind[3];
+  Math::mul3(-1, cfg.state.v, wind);
+  cfg.state.globalToLocal(wind, wind);
+  
+  setFuelFraction(cfg.fuel);
+  setupWeights(cfg.isApproach);
+  
+  // Set up the thruster parameters and iterate until the thrust
+  // stabilizes.
+  for(int i=0; i<_thrusters.size(); i++) {
+    Thruster* t = ((ThrustRec*)_thrusters.get(i))->thruster;
+    t->setWind(wind);
+    t->setStandardAtmosphere(cfg.altitude);
+  }
+  
+  stabilizeThrust();
+  updateGearState();
+  
+  // Precompute thrust in the model, and calculate aerodynamic forces
+  _model.getBody()->recalc();
+  _model.getBody()->reset();
+  _model.initIteration();
+  _model.calcForces(&cfg.state);
 }
-
+/// Used only in Airplane::solve() and solveHelicopter(), not at runtime
 void Airplane::applyDragFactor(float factor)
 {
     float applied = Math::pow(factor, SOLVE_TWEAK);
@@ -971,6 +840,7 @@ void Airplane::applyDragFactor(float factor)
     }
 }
 
+/// Used only in Airplane::solve() and solveHelicopter(), not at runtime
 void Airplane::applyLiftRatio(float factor)
 {
     float applied = Math::pow(factor, SOLVE_TWEAK);
@@ -986,13 +856,7 @@ void Airplane::applyLiftRatio(float factor)
     }
 }
 
-float Airplane::clamp(float val, float min, float max)
-{
-    if(val < min) return min;
-    if(val > max) return max;
-    return val;
-}
-
+/// Helper for solve()
 float Airplane::normFactor(float f)
 {
     if(f < 0) f = -f;
@@ -1015,51 +879,51 @@ void Airplane::solve()
         }
 
 	// Run an iteration at cruise, and extract the needed numbers:
-	runCruise();
+	runConfig(_cruiseConfig);
 
 	_model.getThrust(tmp);
-        float thrust = tmp[0] + _cruiseWeight * Math::sin(_cruiseGlideAngle) * 9.81;
+        float thrust = tmp[0] + _cruiseConfig.weight * Math::sin(_cruiseConfig.glideAngle) * 9.81;
 
 	_model.getBody()->getAccel(tmp);
-        Math::tmul33(_cruiseState.orient, tmp, tmp);
-	float xforce = _cruiseWeight * tmp[0];
-	float clift0 = _cruiseWeight * tmp[2];
+        _cruiseConfig.state.localToGlobal(tmp, tmp);
+	float xforce = _cruiseConfig.weight * tmp[0];
+	float clift0 = _cruiseConfig.weight * tmp[2];
 
 	_model.getBody()->getAngularAccel(tmp);
-        Math::tmul33(_cruiseState.orient, tmp, tmp);
+        _cruiseConfig.state.localToGlobal(tmp, tmp);
 	float pitch0 = tmp[1];
 
 	// Run an approach iteration, and do likewise
-	runApproach();
+        runConfig(_approachConfig);
 
 	_model.getBody()->getAngularAccel(tmp);
-        Math::tmul33(_approachState.orient, tmp, tmp);
+        _approachConfig.state.localToGlobal(tmp, tmp);
 	double apitch0 = tmp[1];
 
 	_model.getBody()->getAccel(tmp);
-        Math::tmul33(_approachState.orient, tmp, tmp);
-	float alift = _approachWeight * tmp[2];
+        _approachConfig.state.localToGlobal(tmp, tmp);
+	float alift = _approachConfig.weight * tmp[2];
 
 	// Modify the cruise AoA a bit to get a derivative
-	_cruiseAoA += ARCMIN;
-	runCruise();
-	_cruiseAoA -= ARCMIN;
-
+	_cruiseConfig.aoa += ARCMIN;
+        runConfig(_cruiseConfig);
+        _cruiseConfig.aoa -= ARCMIN;
+        
 	_model.getBody()->getAccel(tmp);
-        Math::tmul33(_cruiseState.orient, tmp, tmp);
-	float clift1 = _cruiseWeight * tmp[2];
+        _cruiseConfig.state.localToGlobal(tmp, tmp);
+	float clift1 = _cruiseConfig.weight * tmp[2];
 
 	// Do the same with the tail incidence
 	_tail->setIncidence(_tailIncidence + ARCMIN);
-	runCruise();
-	_tail->setIncidence(_tailIncidence);
+        runConfig(_cruiseConfig);
+        _tail->setIncidence(_tailIncidence);
 
 	_model.getBody()->getAngularAccel(tmp);
-        Math::tmul33(_cruiseState.orient, tmp, tmp);
+        _cruiseConfig.state.localToGlobal(tmp, tmp);
 	float pitch1 = tmp[1];
 
 	// Now calculate:
-	float awgt = 9.8f * _approachWeight;
+	float awgt = 9.8f * _approachConfig.weight;
 
 	float dragFactor = thrust / (thrust-xforce);
 	float liftFactor = awgt / (awgt+alift);
@@ -1076,11 +940,11 @@ void Airplane::solve()
         // variable).
         const float ELEVDIDDLE = 0.001f;
         _approachElevator.val += ELEVDIDDLE;
-        runApproach();
+        runConfig(_approachConfig);
         _approachElevator.val -= ELEVDIDDLE;
 
 	_model.getBody()->getAngularAccel(tmp);
-        Math::tmul33(_approachState.orient, tmp, tmp);
+        _approachConfig.state.localToGlobal(tmp, tmp);
 	double apitch1 = tmp[1];
         float elevDelta = -apitch0 * (ELEVDIDDLE/(apitch1-apitch0));
 
@@ -1099,14 +963,14 @@ void Airplane::solve()
 	}
 
 	// OK, now we can adjust the minor variables:
-	_cruiseAoA += SOLVE_TWEAK*aoaDelta;
+	_cruiseConfig.aoa += SOLVE_TWEAK*aoaDelta;
 	_tailIncidence += SOLVE_TWEAK*tailDelta;
 	
-	_cruiseAoA = clamp(_cruiseAoA, -0.175f, 0.175f);
-	_tailIncidence = clamp(_tailIncidence, -0.175f, 0.175f);
+	_cruiseConfig.aoa = Math::clamp(_cruiseConfig.aoa, -0.175f, 0.175f);
+	_tailIncidence = Math::clamp(_tailIncidence, -0.175f, 0.175f);
 
-        if(abs(xforce/_cruiseWeight) < STHRESH*0.0001 &&
-           abs(alift/_approachWeight) < STHRESH*0.0001 &&
+        if(abs(xforce/_cruiseConfig.weight) < STHRESH*0.0001 &&
+           abs(alift/_approachConfig.weight) < STHRESH*0.0001 &&
            abs(aoaDelta) < STHRESH*.000017 &&
            abs(tailDelta) < STHRESH*.000017)
         {
@@ -1129,7 +993,7 @@ void Airplane::solve()
     } else if(_liftRatio < 1e-04 || _liftRatio > 1e4) {
 	_failureMsg = "Lift ratio beyond reasonable bounds.";
 	return;
-    } else if(Math::abs(_cruiseAoA) >= .17453293) {
+    } else if(Math::abs(_cruiseConfig.aoa) >= .17453293) {
 	_failureMsg = "Cruise AoA > 10 degrees";
 	return;
     } else if(Math::abs(_tailIncidence) >= .17453293) {
@@ -1157,14 +1021,22 @@ void Airplane::solveHelicopter()
         applyDragFactor(Math::pow(15.7/1000, 1/SOLVE_TWEAK));
         applyLiftRatio(Math::pow(104, 1/SOLVE_TWEAK));
     }
-    setupState(0,0,0, &_cruiseState);
-    _model.setState(&_cruiseState);
+    _cruiseConfig.state.setupState(0,0,0);
+    _model.setState(&_cruiseConfig.state);
     setupWeights(true);
     _controls.reset();
     _model.getBody()->reset();
-    _model.setAir(_cruiseP, _cruiseT,
-                  Atmosphere::calcStdDensity(_cruiseP, _cruiseT));
-    
+    _model.setStandardAtmosphere(_cruiseConfig.altitude);    
+}
+
+float Airplane::getCGMAC()
+{ 
+    if (_wing) {
+      float cg[3];
+      _model.getBody()->getCG(cg);
+      return (_wing->getMACx() - cg[0]) / _wing->getMAC();
+    }
+    return 0;
 }
 
 }; // namespace yasim
