@@ -21,29 +21,12 @@
 #include "Rotor.hpp"
 #include "Rotorpart.hpp"
 #include "Hitch.hpp"
+#include "Surface.hpp"
 
 #include "FGFDM.hpp"
 
 namespace yasim {
 
-// Some conversion factors
-static const float KTS2MPS = 0.514444444444;
-static const float FT2M = 0.3048;
-static const float DEG2RAD = 0.0174532925199;
-static const float RPM2RAD = 0.10471975512;
-static const float LBS2N = 4.44822;
-static const float LBS2KG = 0.45359237;
-static const float KG2LBS = 2.2046225;
-static const float CM2GALS = 264.172037284;
-static const float HP2W = 745.700;
-static const float INHG2PA = 3386.389;
-static const float K2DEGF = 1.8;
-static const float K2DEGFOFFSET = -459.4;
-static const float CIN2CM = 1.6387064e-5;
-static const float YASIM_PI = 3.14159265358979323846;
-
-static const float NM2FTLB = (1/(LBS2N*FT2M));
-
 // Stubs, so that this can be compiled without the FlightGear
 // binary.  What's the best way to handle this?
 
@@ -59,7 +42,7 @@ FGFDM::FGFDM()
     // Map /controls/flight/elevator to the approach elevator control.  This
     // should probably be settable, but there are very few aircraft
     // who trim their approaches using things other than elevator.
-    _airplane.setElevatorControl(parseAxis("/controls/flight/elevator-trim"));
+    _airplane.setElevatorControl(_airplane.getControlMap()->propertyHandle("/controls/flight/elevator-trim"));
 
     // FIXME: read seed from somewhere?
     int seed = 0;
@@ -68,12 +51,6 @@ FGFDM::FGFDM()
 
 FGFDM::~FGFDM()
 {
-    for(int i=0; i<_axes.size(); i++) {
-        AxisRec* a = (AxisRec*)_axes.get(i);
-        delete[] a->name;
-        delete a;
-    }
-
     for(int i=0; i<_thrusters.size(); i++) {
         EngRec* er = (EngRec*)_thrusters.get(i);
         delete[] er->prefix;
@@ -124,10 +101,40 @@ Airplane* FGFDM::getAirplane()
 
 void FGFDM::init()
 {
+    //reset id generator, needed on simulator reset/re-init
+    Surface::resetIDgen();
     _turb_magnitude_norm = fgGetNode("/environment/turbulence/magnitude-norm", true);
     _turb_rate_hz        = fgGetNode("/environment/turbulence/rate-hz", true);
-    _gross_weight_lbs    = fgGetNode("/yasim/gross-weight-lbs", true);
 
+    _yasimN = fgGetNode("/fdm/yasim", true);
+    _gross_weight_lbs = _yasimN->getNode("gross-weight-lbs", true);
+
+    // alias to older name
+    fgGetNode("/yasim/gross-weight-lbs", true)->alias(_gross_weight_lbs);
+
+    // write some compile time information to property tree
+    _yasimN->getNode("config-version",true)->setIntValue(_airplane.getVersion());
+    _yasimN->getNode("model/cg-x-min",true)->setFloatValue(_airplane.getCGHardLimitXMin());
+    _yasimN->getNode("model/cg-x-max",true)->setFloatValue(_airplane.getCGHardLimitXMax());
+    
+    // prepare nodes for write at runtime
+    _cg_x = _yasimN->getNode("cg-x-m", true);
+    _cg_xmacN = _yasimN->getNode("cg-x-mac", true);
+    _cg_y = _yasimN->getNode("cg-y-m", true);
+    _cg_z = _yasimN->getNode("cg-z-m", true);
+    _vxN = _yasimN->getNode("velocities/v-x", true);
+    _vyN = _yasimN->getNode("velocities/v-y", true);
+    _vzN = _yasimN->getNode("velocities/v-z", true);
+    _vrxN = _yasimN->getNode("velocities/vrot-x", true);
+    _vryN = _yasimN->getNode("velocities/vrot-y", true);
+    _vrzN = _yasimN->getNode("velocities/vrot-z", true);
+    _axN = _yasimN->getNode("accelerations/a-x", true);
+    _ayN = _yasimN->getNode("accelerations/a-y", true);
+    _azN = _yasimN->getNode("accelerations/a-z", true);
+    _arxN = _yasimN->getNode("accelerations/arot-x", true);
+    _aryN = _yasimN->getNode("accelerations/arot-y", true);
+    _arzN = _yasimN->getNode("accelerations/arot-z", true);
+    
     // Allows the user to start with something other than full fuel
     _airplane.setFuelFraction(fgGetFloat("/sim/fuel-fraction", 1));
 
@@ -216,34 +223,68 @@ void FGFDM::startElement(const char* name, const XMLAttributes &atts)
     XMLAttributes* a = (XMLAttributes*)&atts;
     float v[3];
     char buf[64];
-
+    float f = 0;
+    
     if(eq(name, "airplane")) {
-	_airplane.setWeight(attrf(a, "mass") * LBS2KG);
-        if(a->hasAttribute("version")) {
-          _airplane.setVersion( a->getValue("version") );
-        }
-#if defined(ENABLE_DEV_WARNINGS)
-        if( !_airplane.isVersionOrNewer( Version::YASIM_VERSION_CURRENT ) ) {
-          SG_LOG(SG_FLIGHT,SG_ALERT, "This aircraft does not use the latest yasim configuration version.");
-        }
-#endif
+      if(a->hasAttribute("mass")) { f = attrf(a, "mass") * LBS2KG; } 
+      else if (a->hasAttribute("mass-lbs")) { f = attrf(a, "mass-lbs") * LBS2KG; } 
+      else if (a->hasAttribute("mass-kg")) { f = attrf(a, "mass-kg"); }
+      else {
+        SG_LOG(SG_FLIGHT,SG_ALERT,"YASim fatal: missing attribute, airplane needs one of {mass-lbs, mass-kg}");
+        exit(1);       
+      }
+      _airplane.setEmptyWeight(f);
+      if(a->hasAttribute("version")) {
+        _airplane.setVersion( a->getValue("version") );
+      }
+      if( !_airplane.isVersionOrNewer( Version::YASIM_VERSION_CURRENT ) ) {
+        SG_LOG(SG_FLIGHT, SG_DEV_ALERT, "This aircraft does not use the latest yasim configuration version.");
+      }
+      _airplane.setDesiredCGRangeInPercentOfMAC(attrf(a, "cg-min", 0.25f), attrf(a, "cg-max", 0.3f)); //FIXME find reasonable defaults
+      if (attrb(a, "auto-ballast")) {
+        _airplane.setAutoBallast(true);
+      }
     } else if(eq(name, "approach")) {
-	float spd = attrf(a, "speed") * KTS2MPS;
-	float alt = attrf(a, "alt", 0) * FT2M;
-	float aoa = attrf(a, "aoa", 0) * DEG2RAD;
-        float gla = attrf(a, "glide-angle", 0) * DEG2RAD;
-	_airplane.setApproach(spd, alt, aoa, attrf(a, "fuel", 0.2),gla);
-	_cruiseCurr = false;
+      float spd, alt = 0;
+      if (a->hasAttribute("speed")) { spd = attrf(a, "speed") * KTS2MPS; }
+      else if (a->hasAttribute("speed-kt")) { spd = attrf(a, "speed-kt") * KTS2MPS; }
+      else if (a->hasAttribute("speed-kmh")) { spd = attrf(a, "speed-kmh") * KMH2MPS; }
+      else {
+        SG_LOG(SG_FLIGHT,SG_ALERT,"YASim fatal: missing attribute, approach needs one of {speed-kt, speed-kmh}");
+        exit(1);       
+      }
+      if (a->hasAttribute("alt")) { alt = attrf(a, "alt") * FT2M; }
+      else if (a->hasAttribute("alt-ft")) { alt = attrf(a, "alt-ft") * FT2M; }
+      else if (a->hasAttribute("alt-m")) { alt = attrf(a, "alt-m"); }
+      float aoa = attrf(a, "aoa", 0) * DEG2RAD;
+      float gla = attrf(a, "glide-angle", 0) * DEG2RAD;
+      _airplane.setApproach(spd, alt, aoa, attrf(a, "fuel", 0.2), gla);
+      _cruiseCurr = false;
     } else if(eq(name, "cruise")) {
-	float spd = attrf(a, "speed") * KTS2MPS;
-	float alt = attrf(a, "alt") * FT2M;
-        float gla = attrf(a, "glide-angle", 0) * DEG2RAD;
-	_airplane.setCruise(spd, alt, attrf(a, "fuel", 0.5),gla);
-	_cruiseCurr = true;
+      float spd, alt = 0;
+      if (a->hasAttribute("speed")) { spd = attrf(a, "speed") * KTS2MPS; }
+      else if (a->hasAttribute("speed-kt")) { spd = attrf(a, "speed-kt") * KTS2MPS; }
+      else if (a->hasAttribute("speed-kmh")) { spd = attrf(a, "speed-kmh") * KMH2MPS; }
+      else {
+        SG_LOG(SG_FLIGHT,SG_ALERT,"YASim fatal: missing attribute, approach needs one of {speed-kt, speed-kmh}");
+        exit(1);       
+      }
+      if (a->hasAttribute("alt")) { alt = attrf(a, "alt") * FT2M; }
+      else if (a->hasAttribute("alt-ft")) { alt = attrf(a, "alt-ft") * FT2M; }
+      else if (a->hasAttribute("alt-m")) { alt = attrf(a, "alt-m"); }
+      float gla = attrf(a, "glide-angle", 0) * DEG2RAD;
+      _airplane.setCruise(spd, alt, attrf(a, "fuel", 0.5),gla);
+      _cruiseCurr = true;
     } else if(eq(name, "solve-weight")) {
         int idx = attri(a, "idx");
-        float wgt = attrf(a, "weight") * LBS2KG;
-        _airplane.addSolutionWeight(!_cruiseCurr, idx, wgt);
+        if(a->hasAttribute("weight")) { f = attrf(a, "weight") * LBS2KG; }
+        else if(a->hasAttribute("weight-lbs")) { f = attrf(a, "weight-lbs") * LBS2KG; }
+        else if(a->hasAttribute("weight-kg")) { f = attrf(a, "weight-kg"); }
+        else {
+          SG_LOG(SG_FLIGHT,SG_ALERT,"YASim fatal: missing attribute, solve-weight needs one of {weight-lbs, weight-kg}");
+          exit(1);       
+        }
+        _airplane.addSolutionWeight(!_cruiseCurr, idx, f);
     } else if(eq(name, "cockpit")) {
 	v[0] = attrf(a, "x");
 	v[1] = attrf(a, "y");
@@ -293,8 +334,15 @@ void FGFDM::startElement(const char* name, const XMLAttributes &atts)
 	v[0] = attrf(a, "x");
 	v[1] = attrf(a, "y");
 	v[2] = attrf(a, "z");
-	float mass = attrf(a, "mass") * LBS2KG;
-	j->setMaxThrust(attrf(a, "thrust") * LBS2N,
+	float mass;
+        if(a->hasAttribute("mass")) { mass = attrf(a, "mass") * LBS2KG; }
+        else if(a->hasAttribute("mass-lbs")) { mass = attrf(a, "mass-lbs") * LBS2KG; }
+        else if(a->hasAttribute("mass-kg")) { mass = attrf(a, "mass-kg"); }
+        else {
+          SG_LOG(SG_FLIGHT,SG_ALERT,"YASim fatal: missing attribute, jet needs one of {mass-lbs, mass-kg}");
+          exit(1);       
+        }       
+        j->setMaxThrust(attrf(a, "thrust") * LBS2N,
 			attrf(a, "afterburner", 0) * LBS2N);
 	j->setVectorAngle(attrf(a, "rotate", 0) * DEG2RAD);
         j->setReverseThrust(attrf(a, "reverse", 0.2));
@@ -461,12 +509,27 @@ void FGFDM::startElement(const char* name, const XMLAttributes &atts)
 	float density = 6.0; // gasoline, in lbs/gal
 	if(a->hasAttribute("jet")) density = 6.72; 
 	density *= LBS2KG*CM2GALS;
-	_airplane.addTank(v, attrf(a, "capacity") * LBS2KG, density);
+        float capacity = 0;
+        if(a->hasAttribute("capacity")) { capacity = attrf(a, "capacity") * LBS2KG; }
+        else if(a->hasAttribute("capacity-lbs")) { capacity = attrf(a, "capacity-lbs") * LBS2KG; }
+        else if(a->hasAttribute("capacity-kg")) { capacity = attrf(a, "capacity-kg"); }
+        else {
+          SG_LOG(SG_FLIGHT,SG_ALERT,"YASim fatal: missing attribute, tank needs one of {capacity-lbs, capacity-kg}");
+          exit(1);                 
+        }
+        _airplane.addTank(v, capacity, density);
     } else if(eq(name, "ballast")) {
 	v[0] = attrf(a, "x");
 	v[1] = attrf(a, "y");
 	v[2] = attrf(a, "z");
-	_airplane.addBallast(v, attrf(a, "mass") * LBS2KG);
+        if(a->hasAttribute("mass")) { f = attrf(a, "mass") * LBS2KG; } 
+        else if (a->hasAttribute("mass-lbs")) { f = attrf(a, "mass-lbs") * LBS2KG; } 
+        else if (a->hasAttribute("mass-kg")) { f = attrf(a, "mass-kg"); }
+        else {
+          SG_LOG(SG_FLIGHT,SG_ALERT,"YASim fatal: missing attribute, airplane needs one of {mass-lbs, mass-kg}");
+          exit(1);       
+        }
+        _airplane.addBallast(v, f);
     } else if(eq(name, "weight")) {
 	parseWeight(a);
     } else if(eq(name, "stall")) {
@@ -475,16 +538,16 @@ void FGFDM::startElement(const char* name, const XMLAttributes &atts)
 	w->setStallWidth(attrf(a, "width", 2) * DEG2RAD);
 	w->setStallPeak(attrf(a, "peak", 1.5));
     } else if(eq(name, "flap0")) {
-	((Wing*)_currObj)->setFlap0(attrf(a, "start"), attrf(a, "end"),
+	((Wing*)_currObj)->setFlap0Params(attrf(a, "start"), attrf(a, "end"),
 				    attrf(a, "lift"), attrf(a, "drag"));
     } else if(eq(name, "flap1")) {
-	((Wing*)_currObj)->setFlap1(attrf(a, "start"), attrf(a, "end"),
+	((Wing*)_currObj)->setFlap1Params(attrf(a, "start"), attrf(a, "end"),
 				    attrf(a, "lift"), attrf(a, "drag"));
     } else if(eq(name, "slat")) {
-	((Wing*)_currObj)->setSlat(attrf(a, "start"), attrf(a, "end"),
+	((Wing*)_currObj)->setSlatParams(attrf(a, "start"), attrf(a, "end"),
 				   attrf(a, "aoa"), attrf(a, "drag"));
     } else if(eq(name, "spoiler")) {
-	((Wing*)_currObj)->setSpoiler(attrf(a, "start"), attrf(a, "end"),
+	((Wing*)_currObj)->setSpoilerParams(attrf(a, "start"), attrf(a, "end"),
 				      attrf(a, "lift"), attrf(a, "drag"));
     /* } else if(eq(name, "collective")) {
         ((Rotor*)_currObj)->setcollective(attrf(a, "min"), attrf(a, "max"));
@@ -501,33 +564,32 @@ void FGFDM::startElement(const char* name, const XMLAttributes &atts)
 	v[1] = attrf(a, "y");
 	v[2] = attrf(a, "z");
 	((Thruster*)_currObj)->setDirection(v);
-    } else if(eq(name, "control-setting")) {
-	// A cruise or approach control setting
-	const char* axis = a->getValue("axis");
-	float value = attrf(a, "value", 0);
-	if(_cruiseCurr)
-	    _airplane.addCruiseControl(parseAxis(axis), value);
-	else
-	    _airplane.addApproachControl(parseAxis(axis), value);
-    } else if(eq(name, "control-input")) {
-
-	// A mapping of input property to a control
-        int axis = parseAxis(a->getValue("axis"));
-	int control = parseOutput(a->getValue("control"));
-	int opt = 0;
-	opt |= a->hasAttribute("split") ? ControlMap::OPT_SPLIT : 0;
-	opt |= a->hasAttribute("invert") ? ControlMap::OPT_INVERT : 0;
-	opt |= a->hasAttribute("square") ? ControlMap::OPT_SQUARE : 0;
-	
-	ControlMap* cm = _airplane.getControlMap();
-	if(a->hasAttribute("src0")) {
-                           cm->addMapping(axis, control, _currObj, opt,
+  } else if(eq(name, "control-setting")) {
+    // A cruise or approach control setting
+    const char* axis = a->getValue("axis");
+    float value = attrf(a, "value", 0);
+    ControlMap* cm = _airplane.getControlMap();
+    if(_cruiseCurr)
+        _airplane.addCruiseControl(cm->propertyHandle(axis), value);
+    else
+	    _airplane.addApproachControl(cm->propertyHandle(axis), value);
+  } else if(eq(name, "control-input")) {
+    ControlMap* cm = _airplane.getControlMap();
+    // A mapping of input property to a control
+    int axis = cm->propertyHandle(a->getValue("axis"));
+    int control = parseOutput(a->getValue("control"));
+    int opt = 0;
+    opt |= a->hasAttribute("split") ? ControlMap::OPT_SPLIT : 0;
+    opt |= a->hasAttribute("invert") ? ControlMap::OPT_INVERT : 0;
+    opt |= a->hasAttribute("square") ? ControlMap::OPT_SQUARE : 0;
+    if(a->hasAttribute("src0")) {
+       cm->addMapping(axis, control, _currObj, opt,
 			   attrf(a, "src0"), attrf(a, "src1"), 
 			   attrf(a, "dst0"), attrf(a, "dst1"));
-	} else {
-            cm->addMapping(axis, control, _currObj, opt);
-	}
-    } else if(eq(name, "control-output")) {
+    } else {
+      cm->addMapping(axis, control, _currObj, opt);
+    }
+  } else if(eq(name, "control-output")) {
         // A property output for a control on the current object
         ControlMap* cm = _airplane.getControlMap();
         int type = parseOutput(a->getValue("control"));
@@ -568,10 +630,10 @@ void FGFDM::getExternalInput(float dt)
     ControlMap* cm = _airplane.getControlMap();
     cm->reset();
 
-    for(int i=0; i<_axes.size(); i++) {
-        AxisRec* a = (AxisRec*)_axes.get(i);
-        float val = fgGetFloat(a->name, 0);
-        cm->setInput(a->handle, val);
+    for(int i=0; i < cm->numProperties(); i++) {
+        ControlMap::PropHandle *p = cm->getProperty(i);
+        float val = fgGetFloat(p->name, 0);
+        cm->setInput(p->handle, val);
     }
     cm->applyControls(dt);
 
@@ -610,6 +672,33 @@ void FGFDM::setOutputProperties(float dt)
     float grossWgt = _airplane.getModel()->getBody()->getTotalMass() * KG2LBS;
     _gross_weight_lbs->setFloatValue(grossWgt);
 
+    float cg[3];
+    _airplane.getModel()->getBody()->getCG(cg);
+    _cg_x->setFloatValue(cg[0]);
+    _cg_y->setFloatValue(cg[1]);
+    _cg_z->setFloatValue(cg[2]);
+    _cg_xmacN->setFloatValue(_airplane.getCGMAC());
+
+    State* s = _airplane.getModel()->getState();
+    float v[3], acc[3], rot[3], racc[3];
+    Math::vmul33(s->orient, s->v, v);
+    Math::vmul33(s->orient, s->acc, acc);
+    Math::vmul33(s->orient, s->rot, rot);
+    Math::vmul33(s->orient, s->racc, racc);
+
+    _vxN->setFloatValue(v[0]);
+    _vyN->setFloatValue(v[1]);
+    _vzN->setFloatValue(v[2]);
+    _vrxN->setFloatValue(rot[0]);
+    _vryN->setFloatValue(rot[1]);
+    _vrzN->setFloatValue(rot[2]);
+    _axN->setFloatValue(acc[0]);
+    _ayN->setFloatValue(acc[1]);
+    _azN->setFloatValue(acc[2]);
+    _arxN->setFloatValue(racc[0]);
+    _aryN->setFloatValue(racc[1]);
+    _arzN->setFloatValue(racc[2]);
+
     ControlMap* cm = _airplane.getControlMap();
     for(int i=0; i<_controlProps.size(); i++) {
         PropOut* p = (PropOut*)_controlProps.get(i);
@@ -711,32 +800,40 @@ void FGFDM::setOutputProperties(float dt)
 
 Wing* FGFDM::parseWing(XMLAttributes* a, const char* type, Version * version)
 {
-    Wing* w = new Wing(version);
-
     float defDihed = 0;
-    if(eq(type, "vstab"))
-	defDihed = 90;
-    else
-	w->setMirror(true);
+    bool mirror = true;
 
-    float pos[3];
-    pos[0] = attrf(a, "x");
-    pos[1] = attrf(a, "y");
-    pos[2] = attrf(a, "z");
-    w->setBase(pos);
+    if(eq(type, "vstab")) {
+      defDihed = 90;
+      mirror = false;
+    }
 
-    w->setLength(attrf(a, "length"));
-    w->setChord(attrf(a, "chord"));
-    w->setSweep(attrf(a, "sweep", 0) * DEG2RAD);
-    w->setTaper(attrf(a, "taper", 1));
-    w->setDihedral(attrf(a, "dihedral", defDihed) * DEG2RAD);
-    w->setCamber(attrf(a, "camber", 0));
+    float base[3];
+    base[0] = attrf(a, "x");
+    base[1] = attrf(a, "y");
+    base[2] = attrf(a, "z");    
+
+    float length = attrf(a, "length");
+    float chord = attrf(a, "chord");
+    float sweep = attrf(a, "sweep", 0) * DEG2RAD;
+    float taper = attrf(a, "taper", 1);
+    float dihedral = attrf(a, "dihedral", defDihed) * DEG2RAD;
 
     // These come in with positive indicating positive AoA, but the
     // internals expect a rotation about the left-pointing Y axis, so
     // invert the sign.
-    w->setIncidence(attrf(a, "incidence", 0) * DEG2RAD * -1);
-    w->setTwist(attrf(a, "twist", 0) * DEG2RAD * -1);
+    float incidence = attrf(a, "incidence", 0) * DEG2RAD * -1;
+    float twist = attrf(a, "twist", 0) * DEG2RAD * -1;
+
+    float camber = attrf(a, "camber", 0);
+    if (!version->isVersionOrNewer(Version::YASIM_VERSION_2017_2) && (camber == 0)) {
+      SG_LOG(SG_FLIGHT, SG_DEV_WARN, "YASIM warning: versions before 2017.2 are buggy for wings with camber=0");
+    }
+    
+    Wing* w = new Wing(version, mirror, base, chord, length, 
+                       taper, sweep, dihedral, twist);
+    w->setIncidence(incidence);
+    w->setCamber(camber);
 
     // The 70% is a magic number that sorta kinda seems to match known
     // throttle settings to approach speed.
@@ -992,26 +1089,7 @@ void FGFDM::parsePropeller(XMLAttributes* a)
     _currObj = thruster;
 }
 
-// Turns a string axis name into an integer for use by the
-// ControlMap.  Creates a new axis if this one hasn't been defined
-// yet.
-int FGFDM::parseAxis(const char* name)
-{
-    for(int i=0; i<_axes.size(); i++) {
-        AxisRec* a = (AxisRec*)_axes.get(i);
-        if(eq(a->name, name))
-            return a->handle;
-    }
-
-    // Not there, make a new one.
-    AxisRec* a = new AxisRec();
-    a->name = dup(name);
-    fgGetNode( a->name, true ); // make sure the property name exists
-    a->handle = _airplane.getControlMap()->newInput();
-    _axes.add(a);
-    return a->handle;
-}
-
+/// map identifier (string) to int (enum in ControlMap)
 int FGFDM::parseOutput(const char* name)
 {
     if(eq(name, "THROTTLE"))  return ControlMap::THROTTLE;
