@@ -29,23 +29,6 @@ float fgGetFloat (const char * name, float defaultValue) { return 0; }
 double fgGetDouble (const char * name, double defaultValue) { return 0; }
 bool fgSetDouble (const char * name, double defaultValue) { return false; }
 
-static const float KTS2MPS = 0.514444444444;
-static const float RPM2RAD = 0.10471975512;
-static const float HP2W = 745.700;
-static const float FT2M = 0.3048;
-static const float N2LB = 0.224809;
-
-// static const float DEG2RAD = 0.0174532925199;
-// static const float LBS2N = 4.44822;
-// static const float LBS2KG = 0.45359237;
-// static const float KG2LBS = 2.2046225;
-// static const float CM2GALS = 264.172037284;
-// static const float INHG2PA = 3386.389;
-// static const float K2DEGF = 1.8;
-// static const float K2DEGFOFFSET = -459.4;
-// static const float CIN2CM = 1.6387064e-5;
-// static const float YASIM_PI = 3.14159265358979323846;
-
 const int COUNT = 100;
 
 int main(int argc, char** argv)
@@ -71,9 +54,7 @@ int main(int argc, char** argv)
     eng->setBoost(1);
 
     float alt = (argc > 2 ? atof(argv[2]) : 0) * FT2M;
-    pe->setAir(Atmosphere::getStdPressure(alt),
-               Atmosphere::getStdTemperature(alt),
-               Atmosphere::getStdDensity(alt));
+    pe->setStandardAtmosphere(alt);
  
     float speed = (argc > 3 ? atof(argv[3]) : 0) * KTS2MPS;
     float wind[3];
