@@ -67,7 +67,6 @@
 
 #include <Main/globals.hxx>
 #include <Scenery/scenery.hxx>
-#include <Scenery/tilemgr.hxx>
 
 #include "flight.hxx"
 
@@ -330,8 +329,8 @@ FGGroundCache::prepare_ground_cache(double startSimTime, double endSimTime,
     SGGeod geodPt = SGGeod::fromCart(pt);
     // Don't blow away the cache ground_radius and stuff if there's no
     // scenery
-    if (!globals->get_tile_mgr()->schedule_scenery(geodPt, rad, 1.0)) {
-        SG_LOG(SG_FLIGHT, SG_WARN, "prepare_ground_cache(): scenery_available "
+    if (!globals->get_scenery()->schedule_scenery(geodPt, rad, 1.0)) {
+        SG_LOG(SG_FLIGHT, SG_BULK, "prepare_ground_cache(): scenery_available "
                "returns false at " << geodPt << " " << pt << " " << rad);
         return false;
     }
