@@ -40,28 +40,18 @@
 // FlightGear
 #include <Main/globals.hxx>
 
-
 const int STANDARD_THUMBNAIL_HEIGHT = 128;
 const int STANDARD_THUMBNAIL_WIDTH = 172;
+static quint32 CACHE_VERSION = 7;
 
 using namespace simgear::pkg;
 
-AircraftItem::AircraftItem() :
-    excluded(false),
-    usesHeliports(false),
-    usesSeaports(false)
+AircraftItem::AircraftItem()
 {
-    // oh for C++11 initialisers
-    for (int i=0; i<4; ++i) ratings[i] = 0;
 }
 
-AircraftItem::AircraftItem(QDir dir, QString filePath) :
-    excluded(false),
-    usesHeliports(false),
-    usesSeaports(false)
+AircraftItem::AircraftItem(QDir dir, QString filePath)
 {
-    for (int i=0; i<4; ++i) ratings[i] = 0;
-
     SGPropertyNode root;
     readProperties(filePath.toStdString(), &root);
 
@@ -97,6 +87,12 @@ AircraftItem::AircraftItem(QDir dir, QString filePath) :
 
     if (sim->hasChild("variant-of")) {
         variantOf = sim->getStringValue("variant-of");
+    } else {
+        isPrimary = true;
+    }
+
+    if (sim->hasChild("primary-set")) {
+        isPrimary = sim->getBoolValue("primary-set");
     }
 
     if (sim->hasChild("tags")) {
@@ -113,6 +109,22 @@ AircraftItem::AircraftItem(QDir dir, QString filePath) :
             }
         } // of tags iteration
     } // of set-xml has tags
+
+    if (sim->hasChild("previews")) {
+        SGPropertyNode_ptr previewsNode = sim->getChild("previews");
+        for (auto previewNode : previewsNode->getChildren("preview")) {
+            // add file path as url
+            QString pathInXml = QString::fromStdString(previewNode->getStringValue("path"));
+            QString previewPath = dir.absoluteFilePath(pathInXml);
+            previews.append(QUrl::fromLocalFile(previewPath));
+        }
+    }
+
+    if (sim->hasChild("thumbnail")) {
+        thumbnailPath = sim->getStringValue("thumbnail");
+    } else {
+        thumbnailPath = "thumbnail.jpg";
+    }
 }
 
 QString AircraftItem::baseName() const
@@ -129,8 +141,10 @@ void AircraftItem::fromDataStream(QDataStream& ds)
         return;
     }
 
-    ds >> description >> longDescription >> authors >> variantOf;
+    ds >> description >> longDescription >> authors >> variantOf >> isPrimary;
     for (int i=0; i<4; ++i) ds >> ratings[i];
+    ds >> previews;
+    ds >> thumbnailPath;
 }
 
 void AircraftItem::toDataStream(QDataStream& ds) const
@@ -140,20 +154,22 @@ void AircraftItem::toDataStream(QDataStream& ds) const
         return;
     }
 
-    ds << description << longDescription << authors << variantOf;
+    ds << description << longDescription << authors << variantOf << isPrimary;
     for (int i=0; i<4; ++i) ds << ratings[i];
+    ds << previews;
+    ds << thumbnailPath;
 }
 
-QPixmap AircraftItem::thumbnail() const
+QPixmap AircraftItem::thumbnail(bool loadIfRequired) const
 {
-    if (m_thumbnail.isNull()) {
+    if (m_thumbnail.isNull() && loadIfRequired) {
         QFileInfo info(path);
         QDir dir = info.dir();
-        if (dir.exists("thumbnail.jpg")) {
-            m_thumbnail.load(dir.filePath("thumbnail.jpg"));
+        if (dir.exists(thumbnailPath)) {
+            m_thumbnail.load(dir.filePath(thumbnailPath));
             // resize to the standard size
             if (m_thumbnail.height() > STANDARD_THUMBNAIL_HEIGHT) {
-                m_thumbnail = m_thumbnail.scaledToHeight(STANDARD_THUMBNAIL_HEIGHT);
+                m_thumbnail = m_thumbnail.scaledToHeight(STANDARD_THUMBNAIL_HEIGHT, Qt::SmoothTransformation);
             }
         }
     }
@@ -161,9 +177,6 @@ QPixmap AircraftItem::thumbnail() const
     return m_thumbnail;
 }
 
-
-static quint32 CACHE_VERSION = 5;
-
 class AircraftScanThread : public QThread
 {
     Q_OBJECT
@@ -284,7 +297,7 @@ private:
                         continue;
                     }
 
-                    if (item->variantOf.isNull()) {
+                    if (item->isPrimary) {
                         baseAircraft.insert(item->baseName(), item);
                     } else {
                         variants.append(item);
@@ -400,18 +413,25 @@ protected:
 
         QPixmap pix = QPixmap::fromImage(img);
         if (pix.height() > STANDARD_THUMBNAIL_HEIGHT) {
-            pix = pix.scaledToHeight(STANDARD_THUMBNAIL_HEIGHT);
+            pix = pix.scaledToHeight(STANDARD_THUMBNAIL_HEIGHT, Qt::SmoothTransformation);
         }
-        m_model->m_thumbnailPixmapCache.insert(QString::fromStdString(aThumbnailUrl), pix);
 
-        // notify any affected items. Linear scan here avoids another map/dict
-        // structure.
-        for (auto it=m_model->m_packages.begin(); it != m_model->m_packages.end(); ++it) {
-            const string_list& urls((*it)->thumbnailUrls());
-            auto cit = std::find(urls.begin(), urls.end(), aThumbnailUrl);
-            if (cit != urls.end()) {
-                size_t offset = it - m_model->m_packages.begin();
-                QModelIndex mi(m_model->index(offset + m_model->m_items.size()));
+        m_model->m_downloadedPixmapCache.insert(QString::fromStdString(aThumbnailUrl), pix);
+
+        // notify any affected items. Linear scan here avoids another map/dict structure.
+        for (auto pkg : m_model->m_packages) {
+            const size_t variantCount = pkg->variants().size();
+            bool notifyChanged = false;
+
+            for (size_t v=0; v < variantCount; ++v) {
+                const Package::Thumbnail& thumb(pkg->thumbnailForVariant(v));
+                if (thumb.url == aThumbnailUrl) {
+                    notifyChanged = true;
+                }
+            }
+
+            if (notifyChanged) {
+                QModelIndex mi = indexForPackage(pkg);
                 m_model->dataChanged(mi, mi);
             }
         } // of packages iteration
@@ -426,14 +446,14 @@ private:
             return QModelIndex();
         }
 
-        size_t offset = it - m_model->m_packages.begin();
+        int offset = std::distance(m_model->m_packages.begin(), it);
         return m_model->index(offset + m_model->m_items.size());
     }
 
     AircraftItemModel* m_model;
 };
 
-AircraftItemModel::AircraftItemModel(QObject* pr ) :
+AircraftItemModel::AircraftItemModel(QObject* pr) :
     QAbstractListModel(pr)
 {
 }
@@ -594,10 +614,6 @@ QVariant AircraftItemModel::data(const QModelIndex& index, int role) const
         return m_delegateStates.at(row).variant;
     }
 
-    if (role == AircraftCurrentThumbnailRole) {
-        return m_delegateStates.at(row).thumbnail;
-    }
-
     if (row >= m_items.size()) {
         quint32 packageIndex = row - m_items.size();
         const PackageRef& pkg(m_packages[packageIndex]);
@@ -607,6 +623,8 @@ QVariant AircraftItemModel::data(const QModelIndex& index, int role) const
             return ex.valid() ? ex->downloadedPercent() : 0;
         } else if (role == AircraftInstallDownloadedSizeRole) {
             return static_cast<quint64>(ex.valid() ? ex->downloadedBytes() : 0);
+        } else if (role == AircraftPackageRefRole ) {
+            return QVariant::fromValue(pkg);
         }
 
         return dataFromPackage(pkg, m_delegateStates.at(row), role);
@@ -622,16 +640,7 @@ QVariant AircraftItemModel::dataFromItem(AircraftItemPtr item, const DelegateSta
         return item->variants.count();
     }
 
-    if (role == AircraftThumbnailCountRole) {
-        QPixmap p = item->thumbnail();
-        return p.isNull() ? 0 : 1;
-    }
-
-    if (role == AircraftThumbnailSizeRole) {
-        return item->thumbnail().size();
-    }
-
-    if ((role >= AircraftVariantDescriptionRole) && (role < AircraftThumbnailRole)) {
+    if (role >= AircraftVariantDescriptionRole) {
         int variantIndex = role - AircraftVariantDescriptionRole;
         return item->variants.at(variantIndex)->description;
     }
@@ -643,6 +652,14 @@ QVariant AircraftItemModel::dataFromItem(AircraftItemPtr item, const DelegateSta
         }
     }
 
+    if (role == AircraftThumbnailSizeRole) {
+        QPixmap pm = item->thumbnail(false);
+        if (pm.isNull()) {
+            return QSize(STANDARD_THUMBNAIL_WIDTH, STANDARD_THUMBNAIL_HEIGHT);
+        }
+        return pm.size();
+    }
+
     if (role == Qt::DisplayRole) {
         if (item->description.isEmpty()) {
             return tr("Missing description for: %1").arg(item->baseName());
@@ -657,7 +674,13 @@ QVariant AircraftItemModel::dataFromItem(AircraftItemPtr item, const DelegateSta
         return item->authors;
     } else if ((role >= AircraftRatingRole) && (role < AircraftVariantDescriptionRole)) {
         return item->ratings[role - AircraftRatingRole];
-    } else if (role >= AircraftThumbnailRole) {
+    } else if (role == AircraftPreviewsRole) {
+        QVariantList result;
+        Q_FOREACH(QUrl u, item->previews) {
+            result.append(u);
+        }
+        return result;
+    } else if (role == AircraftThumbnailRole) {
         return item->thumbnail();
     } else if (role == AircraftPackageIdRole) {
         // can we fake an ID? otherwise fall through to a null variant
@@ -687,10 +710,10 @@ QVariant AircraftItemModel::dataFromItem(AircraftItemPtr item, const DelegateSta
 QVariant AircraftItemModel::dataFromPackage(const PackageRef& item, const DelegateState& state, int role) const
 {
     if (role == Qt::DecorationRole) {
-        role = AircraftThumbnailRole; // use first thumbnail
+        role = AircraftThumbnailRole;
     }
 
-    if ((role >= AircraftVariantDescriptionRole) && (role < AircraftThumbnailRole)) {
+    if (role >= AircraftVariantDescriptionRole) {
         int variantIndex = role - AircraftVariantDescriptionRole;
         QString desc = QString::fromStdString(item->nameForVariant(variantIndex));
         if (desc.isEmpty()) {
@@ -738,18 +761,18 @@ QVariant AircraftItemModel::dataFromPackage(const PackageRef& item, const Delega
         if (pm.isNull())
             return QSize(STANDARD_THUMBNAIL_WIDTH, STANDARD_THUMBNAIL_HEIGHT);
         return pm.size();
-    } else if (role >= AircraftThumbnailRole) {
-        DelegateState changedState(state);
-        // override the current thumbnail as required
-        changedState.thumbnail = (role - AircraftThumbnailRole);
-        return packageThumbnail(item, changedState);
+    } else if (role == AircraftThumbnailRole) {
+        return packageThumbnail(item, state);
+    } else if (role == AircraftPreviewsRole) {
+        return packagePreviews(item, state);
     } else if (role == AircraftAuthorsRole) {
-        SGPropertyNode* authors = item->properties()->getChild("author");
-        if (authors) {
-            return QString::fromStdString(authors->getStringValue());
+        std::string authors = item->getLocalisedProp("author", state.variant);
+        if (!authors.empty()) {
+            return QString::fromStdString(authors);
         }
     } else if (role == AircraftLongDescriptionRole) {
-        return QString::fromStdString(item->description()).simplified();
+        std::string longDesc = item->getLocalisedProp("description", state.variant);
+        return QString::fromStdString(longDesc).simplified();
     } else if (role == AircraftPackageSizeRole) {
         return static_cast<int>(item->fileSizeBytes());
     } else if (role == AircraftURIRole) {
@@ -770,46 +793,69 @@ QVariant AircraftItemModel::dataFromPackage(const PackageRef& item, const Delega
 
 QVariant AircraftItemModel::packageThumbnail(PackageRef p, const DelegateState& ds, bool download) const
 {
-    const string_list& thumbnails(p->thumbnailUrls());
-    if (ds.thumbnail >= static_cast<int>(thumbnails.size())) {
+    const Package::Thumbnail& thumb(p->thumbnailForVariant(ds.variant));
+    if (thumb.url.empty()) {
         return QVariant();
     }
 
-    std::string thumbnailUrl = thumbnails.at(ds.thumbnail);
-    QString urlQString(QString::fromStdString(thumbnailUrl));
-    if (m_thumbnailPixmapCache.contains(urlQString)) {
+    QString urlQString(QString::fromStdString(thumb.url));
+    if (m_downloadedPixmapCache.contains(urlQString)) {
         // cache hit, easy
-        return m_thumbnailPixmapCache.value(urlQString);
+        return m_downloadedPixmapCache.value(urlQString);
     }
 
-// check the on-disk store. This relies on the order of thumbnails in the
-// results of thumbnailUrls and thumbnails corresponding
+// check the on-disk store.
     InstallRef ex = p->existingInstall();
     if (ex.valid()) {
-        const string_list& thumbNames(p->thumbnails());
-        if (!thumbNames.empty()) {
-            SGPath path(ex->path());
-            path.append(p->thumbnails()[ds.thumbnail]);
-            if (path.exists()) {
-                QPixmap pix;
-                pix.load(QString::fromStdString(path.utf8Str()));
-                // resize to the standard size
-                if (pix.height() > STANDARD_THUMBNAIL_HEIGHT) {
-                    pix = pix.scaledToHeight(STANDARD_THUMBNAIL_HEIGHT);
-                }
-                m_thumbnailPixmapCache[urlQString] = pix;
-                return pix;
+        SGPath thumbPath = ex->path() / thumb.path;
+        if (thumbPath.exists()) {
+            QPixmap pix;
+            pix.load(QString::fromStdString(thumbPath.utf8Str()));
+            // resize to the standard size
+            if (pix.height() > STANDARD_THUMBNAIL_HEIGHT) {
+                pix = pix.scaledToHeight(STANDARD_THUMBNAIL_HEIGHT);
             }
-        } // of have thumbnail file names
+            m_downloadedPixmapCache[urlQString] = pix;
+            return pix;
+        }
     } // of have existing install
 
     if (download) {
-        m_packageRoot->requestThumbnailData(thumbnailUrl);
+        m_packageRoot->requestThumbnailData(thumb.url);
     }
 
     return QVariant();
 }
 
+QVariant AircraftItemModel::packagePreviews(PackageRef p, const DelegateState& ds) const
+{
+    const Package::PreviewVec& previews = p->previewsForVariant(ds.variant);
+    if (previews.empty()) {
+        return QVariant();
+    }
+
+    QVariantList result;
+    // if we have an install, return file URLs, not remote (http) ones
+    InstallRef ex = p->existingInstall();
+    if (ex.valid()) {
+        for (auto p : previews) {
+            SGPath localPreviewPath = ex->path() / p.path;
+            if (!localPreviewPath.exists()) {
+                qWarning() << "missing local preview" << QString::fromStdString(localPreviewPath.utf8Str());
+                continue;
+            }
+            result.append(QUrl::fromLocalFile(QString::fromStdString(localPreviewPath.utf8Str())));
+        }
+    }
+
+    // return remote urls
+    for (auto p : previews) {
+        result.append(QUrl(QString::fromStdString(p.url)));
+    }
+
+    return result;
+}
+
 bool AircraftItemModel::setData(const QModelIndex &index, const QVariant &value, int role)
   {
       int row = index.row();
@@ -825,16 +871,6 @@ bool AircraftItemModel::setData(const QModelIndex &index, const QVariant &value,
           return true;
       }
 
-      if (role == AircraftCurrentThumbnailRole) {
-          if (m_delegateStates[row].thumbnail == newValue) {
-              return true;
-          }
-
-          m_delegateStates[row].thumbnail = newValue;
-          emit dataChanged(index, index);
-          return true;
-      }
-
       return false;
   }
 
@@ -928,12 +964,42 @@ void AircraftItemModel::selectVariantForAircraftURI(QUrl uri)
     }
 
     if (modelIndex.isValid()) {
-        qDebug() << "selected variant index" << variantIndex << "at" << modelIndex
-        << "for variant" << uri;
         setData(modelIndex, variantIndex, AircraftVariantRole);
     }
 }
 
+QString AircraftItemModel::nameForAircraftURI(QUrl uri) const
+{
+    if (uri.isLocalFile()) {
+        QString path = uri.toLocalFile();
+        for (int row=0; row <m_items.size(); ++row) {
+            const AircraftItemPtr item(m_items.at(row));
+            if (item->path == path) {
+                return item->description;
+            }
+
+            // check variants too
+            for (int vr=0; vr < item->variants.size(); ++vr) {
+                if (item->variants.at(vr)->path == path) {
+                    return item->description;
+                }
+            }
+        }
+    } else if (uri.scheme() == "package") {
+        QString ident = uri.path();
+        PackageRef pkg = m_packageRoot->getPackageById(ident.toStdString());
+        if (pkg) {
+            int variantIndex = pkg->indexOfVariant(ident.toStdString());
+            return QString::fromStdString(pkg->nameForVariant(variantIndex));
+        }
+    } else {
+        qWarning() << "Unknown aircraft URI scheme" << uri << uri.scheme();
+        return QString();
+    }
+
+    return QString();
+}
+
 void AircraftItemModel::onScanResults()
 {
     QVector<AircraftItemPtr> newItems = m_scanThread->items();
