@@ -10,8 +10,8 @@
 #include "CatalogListModel.hxx"
 #include "AddCatalogDialog.hxx"
 #include "AircraftModel.hxx"
-#include "QtLauncher_private.hxx"
 #include "InstallSceneryDialog.hxx"
+#include "QtLauncher.hxx"
 
 #include <Main/options.hxx>
 #include <Main/globals.hxx>
@@ -53,26 +53,12 @@ AddOnsPage::AddOnsPage(QWidget *parent, simgear::pkg::RootRef root) :
     connect(m_ui->removeAircraftPath, &QToolButton::clicked,
             this, &AddOnsPage::onRemoveAircraftPath);
 
-    connect(m_ui->changeDownloadDir, &QPushButton::clicked,
-            this, &AddOnsPage::onChangeDownloadDir);
-
-    connect(m_ui->clearDownloadDir, &QPushButton::clicked,
-            this, &AddOnsPage::onClearDownloadDir);
-
-    connect(m_ui->changeDataDir, &QPushButton::clicked,
-            this, &AddOnsPage::onChangeDataDir);
     connect(m_ui->installSceneryButton, &QPushButton::clicked,
             this, &AddOnsPage::onInstallScenery);
 
     m_ui->sceneryPathsList->setToolTip(
       tr("After changing this list, please restart the launcher to avoid "
          "possibly inconsistent behavior."));
-    m_ui->changeDownloadDir->setToolTip(
-      tr("After changing this location, you may have to restart the launcher "
-         "to avoid inconsistent behavior."));
-    m_ui->clearDownloadDir->setToolTip(
-      tr("If you use this button, you may have to restart the launcher "
-         "to avoid inconsistent behavior."));
     m_ui->installSceneryButton->setToolTip(
       tr("After installing scenery, you may have to restart the launcher "
          "to avoid inconsistent behavior."));
@@ -82,13 +68,14 @@ AddOnsPage::AddOnsPage(QWidget *parent, simgear::pkg::RootRef root) :
     QStringList sceneryPaths = settings.value("scenery-paths").toStringList();
     m_ui->sceneryPathsList->addItems(sceneryPaths);
 
+    connect(m_ui->sceneryPathsList->model(), &QAbstractItemModel::rowsMoved,
+            this, &AddOnsPage::saveSceneryPaths);
+
     QStringList aircraftPaths = settings.value("aircraft-paths").toStringList();
     m_ui->aircraftPathsList->addItems(aircraftPaths);
 
-    QVariant downloadDir = settings.value("download-dir");
-    if (downloadDir.isValid()) {
-        m_downloadDir = downloadDir.toString();
-    }
+    connect(m_ui->aircraftPathsList->model(), &QAbstractItemModel::rowsMoved,
+            this, &AddOnsPage::saveAircraftPaths);
 
     updateUi();
 }
@@ -102,14 +89,37 @@ void AddOnsPage::onAddSceneryPath()
 {
     QString path = QFileDialog::getExistingDirectory(this, tr("Choose scenery folder"));
     if (!path.isEmpty()) {
+        // validation
+
+        SGPath p(path.toStdString());
+        bool isValid = false;
+
+        for (const auto& dir: {"Objects", "Terrain", "Buildings", "Roads",
+                               "Pylons", "NavData"}) {
+            if ((p / dir).exists()) {
+                isValid = true;
+                break;
+            }
+        }
+
+        if (!isValid) {
+            QMessageBox mb;
+            mb.setText(QString("The folder '%1' doesn't appear to contain scenery - add anyway?").arg(path));
+            mb.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
+            mb.setDefaultButton(QMessageBox::No);
+            mb.setInformativeText(
+                "Added scenery should contain at least one of the following "
+                "folders: Objects, Terrain, Buildings, Roads, Pylons, NavData.");
+            mb.exec();
+
+            if (mb.result() == QMessageBox::No) {
+                return;
+            }
+        }
+
         m_ui->sceneryPathsList->addItem(path);
         saveSceneryPaths();
     }
-
-    // work around a Qt OS-X bug - this dialog is ending ordered
-    // behind the main settings dialog (consequence of modal-dialog
-    // showing a modla dialog showing a modial dialog)
-    window()->raise();
 }
 
 void AddOnsPage::onRemoveSceneryPath()
@@ -157,10 +167,6 @@ void AddOnsPage::onAddAircraftPath()
 
         saveAircraftPaths();
     }
-    // work around a Qt OS-X bug - this dialog is ending ordered
-    // behind the main settings dialog (consequence of modal-dialog
-    // showing a modla dialog showing a modial dialog)
-    window()->raise();
 }
 
 void AddOnsPage::onRemoveAircraftPath()
@@ -181,6 +187,7 @@ void AddOnsPage::saveAircraftPaths()
     }
 
     settings.setValue("aircraft-paths", paths);
+    emit aircraftPathsChanged();
 }
 
 void AddOnsPage::saveSceneryPaths()
@@ -218,13 +225,13 @@ void AddOnsPage::onAddCatalog()
 
 void AddOnsPage::onAddDefaultCatalog()
 {
-    addDefaultCatalog(this);
+    addDefaultCatalog(this, false /* not silent */);
 
     m_catalogsModel->refresh();
     updateUi();
 }
 
-void AddOnsPage::addDefaultCatalog(QWidget* pr)
+void AddOnsPage::addDefaultCatalog(QWidget* pr, bool silent)
 {
     // check it's not a duplicate somehow
     FGHTTPClient* http = globals->get_subsystem<FGHTTPClient>();
@@ -233,6 +240,9 @@ void AddOnsPage::addDefaultCatalog(QWidget* pr)
 
     QScopedPointer<AddCatalogDialog> dlg(new AddCatalogDialog(pr, globals->packageRoot()));
     QUrl url(QString::fromStdString(http->getDefaultCatalogUrl()));
+    if (silent) {
+        dlg->setNonInteractiveMode();
+    }
     dlg->setUrlAndDownload(url);
     dlg->exec();
 
@@ -270,76 +280,11 @@ void AddOnsPage::onRemoveCatalog()
     updateUi();
 }
 
-void AddOnsPage::onChangeDownloadDir()
-{
-    QString path = QFileDialog::getExistingDirectory(this,
-                                                     tr("Choose downloads folder"),
-                                                     m_downloadDir);
-    if (path.isEmpty()) {
-        return; // user cancelled
-    }
-
-    m_downloadDir = path;
-    setDownloadDir();
-}
-
-void AddOnsPage::onClearDownloadDir()
-{
-    // does this need an 'are you sure'?
-    m_downloadDir.clear();
-
-    setDownloadDir();
-}
-
-void AddOnsPage::setDownloadDir()
-{
-    QSettings settings;
-    if (m_downloadDir.isEmpty()) {
-        settings.remove("download-dir");
-    } else {
-        settings.setValue("download-dir", m_downloadDir);
-    }
-
-    if (m_downloadDir.isEmpty()) {
-        flightgear::Options::sharedInstance()->clearOption("download-dir");
-    } else {
-        flightgear::Options::sharedInstance()->setOption("download-dir", m_downloadDir.toStdString());
-    }
-
-    emit downloadDirChanged();
-    updateUi();
-}
-
-void AddOnsPage::onChangeDataDir()
-{
-    QMessageBox mbox(this);
-    mbox.setText(tr("Change the data files used by FlightGear?"));
-    mbox.setInformativeText(tr("FlightGear requires additional files to operate. "
-                               "(Also called the base package, or fg-data) "
-                               "You can restart FlightGear and choose a "
-                               "different data files location, or restore the default setting."));
-    QPushButton* quitButton = mbox.addButton(tr("Restart FlightGear now"), QMessageBox::YesRole);
-    mbox.addButton(QMessageBox::Cancel);
-    mbox.setDefaultButton(QMessageBox::Cancel);
-    mbox.setIconPixmap(QPixmap(":/app-icon-large"));
-
-    mbox.exec();
-    if (mbox.clickedButton() != quitButton) {
-        return;
-    }
-
-    {
-        QSettings settings;
-        // set the option to the magic marker value
-        settings.setValue("fg-root", "!ask");
-    } // scope the ensure settings are written nicely
-
-    QtLauncher::restartTheApp(QStringList());
-}
-
 void AddOnsPage::onInstallScenery()
 {
-    InstallSceneryDialog dlg(this, m_downloadDir);
+    QSettings settings;
+    QString downloadDir = settings.value("download-dir").toString();
+    InstallSceneryDialog dlg(this, downloadDir);
     if (dlg.exec() == QDialog::Accepted) {
         if (!haveSceneryPath(dlg.sceneryPath())) {
             m_ui->sceneryPathsList->addItem(dlg.sceneryPath());
@@ -350,30 +295,11 @@ void AddOnsPage::onInstallScenery()
 
 void AddOnsPage::updateUi()
 {
-    QString s = m_downloadDir;
-    if (s.isEmpty()) {
-        s = QString::fromStdString(flightgear::defaultDownloadDir().utf8Str());
-        s.append(tr(" (default)"));
-        m_ui->clearDownloadDir->setEnabled(false);
-    } else {
-        m_ui->clearDownloadDir->setEnabled(true);
-    }
-
-    QString m = tr("Download location: %1").arg(s);
-    m_ui->downloadLocation->setText(m);
-
-    QString dataLoc;
-    QSettings settings;
-    QString root = settings.value("fg-root").toString();
-    if (root.isNull()) {
-        dataLoc = tr("built-in");
-    } else {
-        dataLoc = root;
-    }
-
-    m_ui->dataLocation->setText(tr("Data location: %1").arg(dataLoc));
-
-
     FGHTTPClient* http = globals->get_subsystem<FGHTTPClient>();
     m_ui->addDefaultCatalogButton->setEnabled(!http->isDefaultCatalogInstalled());
 }
+
+void AddOnsPage::onDraggedAircraftList()
+{
+    qWarning() << "did drag aircraft list";
+}
