@@ -483,6 +483,11 @@ namespace
             return true;
         }
 
+        static void cancel()
+        {
+            _snapShotOp = nullptr;
+        }
+
     private:
         // constructor to be executed in main loop's thread
         GUISnapShotOperation() :
@@ -567,6 +572,11 @@ bool fgDumpSnapShot ()
     return GUISnapShotOperation::start();
 }
 
+void fgCancelSnapShot()
+{
+    GUISnapShotOperation::cancel();
+}
+
 // do an entire scenegraph dump
 void fgDumpSceneGraph()
 {
