@@ -58,7 +58,7 @@
 #include <simgear/structure/SGPerfMon.hxx>
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/misc/sg_dir.hxx>
-#include <simgear/misc/sgstream.hxx>
+#include <simgear/io/iostreams/sgstream.hxx>
 #include <simgear/misc/strutils.hxx>
 #include <simgear/props/props_io.hxx>
 #include <simgear/scene/tsync/terrasync.hxx>
@@ -111,7 +111,6 @@
 #include <Navaids/navlist.hxx>
 #include <Scenery/scenery.hxx>
 #include <Scenery/SceneryPager.hxx>
-#include <Scenery/tilemgr.hxx>
 #include <Scripting/NasalSys.hxx>
 #include <Sound/voice.hxx>
 #include <Sound/soundmanager.hxx>
@@ -131,6 +130,7 @@
 #include <Instrumentation/HUD/HUD.hxx>
 #include <Cockpit/cockpitDisplayManager.hxx>
 #include <Network/HTTPClient.hxx>
+#include <Network/DNSClient.hxx>
 #include <Network/fgcom.hxx>
 #include <Network/http/httpd.hxx>
 #include <Include/version.h>
@@ -195,7 +195,8 @@ public:
   {
     std::string aircraft = fgGetString( "/sim/aircraft", "");
     if (aircraft.empty()) {
-        flightgear::fatalMessageBox("No aircraft", "No aircraft was specified");
+      flightgear::fatalMessageBoxWithoutExit("No aircraft",
+                                             "No aircraft was specified");
       SG_LOG(SG_GENERAL, SG_ALERT, "no aircraft specified");
       return false;
     }
@@ -212,10 +213,12 @@ public:
         try {
           readProperties(setFile, globals->get_props());
         } catch ( const sg_exception &e ) {
-          SG_LOG(SG_INPUT, SG_ALERT, "Error reading aircraft: " << e.getFormattedMessage());
-            flightgear::fatalMessageBox("Error reading aircraft",
-                                        "An error occured reading the requested aircraft (" + aircraft + ")",
-                                        e.getFormattedMessage());
+          SG_LOG(SG_INPUT, SG_ALERT,
+                 "Error reading aircraft: " << e.getFormattedMessage());
+          flightgear::fatalMessageBoxWithoutExit(
+            "Error reading aircraft",
+            "An error occured reading the requested aircraft (" + aircraft + ")",
+            e.getFormattedMessage());
           return false;
         }
           // apply state after the -set.xml, but before any options are are set
@@ -224,9 +227,11 @@ public:
       } else {
         SG_LOG(SG_GENERAL, SG_ALERT, "aircraft '" << _searchAircraft << 
                "' not found in specified dir:" << aircraftDir);
-          flightgear::fatalMessageBox("Aircraft not found",
-                                      "The requested aircraft '" + aircraft + "' could not be found in the specified location.",
-                                      aircraftDir);
+        flightgear::fatalMessageBoxWithoutExit(
+          "Aircraft not found",
+          "The requested aircraft (" + aircraft + ") could not be found "
+          "in the specified location.",
+          aircraftDir);
         return false;
       }
     }
@@ -245,10 +250,12 @@ public:
     }
     
     if (_foundPath.isNull()) {
-      SG_LOG(SG_GENERAL, SG_ALERT, "Cannot find specified aircraft: " << aircraft );
-        flightgear::fatalMessageBox("Aircraft not found",
-                                    "The requested aircraft '" + aircraft + "' could not be found in any of the search paths");
-
+      SG_LOG(SG_GENERAL, SG_ALERT,
+             "Cannot find the specified aircraft: '" << aircraft << "'");
+      flightgear::fatalMessageBoxWithoutExit(
+        "Aircraft not found",
+        "The requested aircraft (" + aircraft + ") could not be found "
+        "in any of the search paths.");
       return false;
     }
     
@@ -262,10 +269,12 @@ public:
     try {
       readProperties(_foundPath, globals->get_props());
     } catch ( const sg_exception &e ) {
-      SG_LOG(SG_INPUT, SG_ALERT, "Error reading aircraft: " << e.getFormattedMessage());
-        flightgear::fatalMessageBox("Error reading aircraft",
-                                    "An error occured reading the requested aircraft (" + aircraft + ")",
-                                    e.getFormattedMessage());
+      SG_LOG(SG_INPUT, SG_ALERT,
+             "Error reading aircraft: " << e.getFormattedMessage());
+      flightgear::fatalMessageBoxWithoutExit(
+        "Error reading aircraft",
+        "An error occured reading the requested aircraft (" + aircraft + ")",
+        e.getFormattedMessage());
       return false;
     }
 
@@ -348,10 +357,17 @@ private:
 #ifdef _WIN32
 static SGPath platformDefaultDataPath()
 {
-  char *envp = ::getenv( "APPDATA" );
-  SGPath config( envp );
-  config.append( "flightgear.org" );
-  return config;
+  SGPath appDataPath = SGPath::fromEnv("APPDATA");
+
+  if (appDataPath.isNull()) {
+    flightgear::fatalMessageBoxThenExit(
+      "FlightGear", "Unable to get the value of APPDATA.",
+      "FlightGear is unable to retrieve the value of the APPDATA environment "
+      "variable. This is quite unexpected on Windows platforms, and FlightGear "
+      "can't continue its execution without this value, sorry.");
+  }
+
+  return appDataPath / "flightgear.org";
 }
 
 #elif defined(SG_MAC)
@@ -376,9 +392,10 @@ bool fgInitHome()
     }
     
     if (!fgHome.exists()) {
-        flightgear::fatalMessageBox("Problem setting up user data",
-                                    "Unable to create the user-data storage folder at: '"
-                                    + dataPath.utf8Str() + "'");
+        flightgear::fatalMessageBoxWithoutExit(
+          "Problem setting up user data",
+          "Unable to create the user-data storage folder at '" +
+          dataPath.utf8Str() + "'.");
         return false;
     }
     
@@ -388,6 +405,26 @@ bool fgInitHome()
         return true;
     }
     
+	bool result = false;
+#if defined(SG_WINDOWS)
+	// don't use a PID file on Windows, because deleting on close is
+	// unreliable and causes false-positives. Instead, use a named
+	// mutex.
+
+	HANDLE hMutex = CreateMutexA(nullptr, FALSE, "org.flightgear.fgfs.primary");
+	if (hMutex == nullptr) {
+		printf("CreateMutex error: %d\n", GetLastError());
+		SG_LOG(SG_GENERAL, SG_POPUP, "Failed to create mutex for multi-app protection");
+		return false;
+	} else if (GetLastError() == ERROR_ALREADY_EXISTS) {
+		SG_LOG(SG_GENERAL, SG_ALERT, "flightgear instance already running, switching to FG_HOME read-only.");
+		fgSetBool("/sim/fghome-readonly", true);
+		return true;
+	} else {
+		SG_LOG(SG_GENERAL, SG_INFO, "Created multi-app mutex, we are in writeable mode");
+		result = true;
+	}
+#else
 // write our PID, and check writeability
     SGPath pidPath(dataPath, "fgfs.pid");
     if (pidPath.exists()) {
@@ -397,27 +434,11 @@ bool fgInitHome()
         fgSetBool("/sim/fghome-readonly", true);
         return true;
     }
-    
-    char buf[16];
-    bool result = false;
-    std::string ps = pidPath.local8BitStr();
-#if defined(SG_WINDOWS)
-    size_t len = snprintf(buf, 16, "%d", _getpid());
 
-    HANDLE f = CreateFileA(ps.c_str(), GENERIC_READ | GENERIC_WRITE,
-						   FILE_SHARE_READ, /* sharing */
-                           NULL, /* security attributes */
-                           CREATE_NEW, /* error if already exists */
-                           FILE_FLAG_DELETE_ON_CLOSE,
-						   NULL /* template */);
-    
-    result = (f != INVALID_HANDLE_VALUE);
-    if (result) {
-		DWORD written;
-        WriteFile(f, buf, len, &written, NULL /* overlapped */);
-    }
-#else
-    // POSIX, do open+unlink trick to the file is deleted on exit, even if we
+    char buf[16];
+    std::string ps = pidPath.local8BitStr();
+
+    // do open+unlink trick to the file is deleted on exit, even if we
     // crash or exit(-1)
     ssize_t len = snprintf(buf, 16, "%d", getpid());
     int fd = ::open(ps.c_str(), O_WRONLY | O_CREAT | O_TRUNC | O_EXCL, 0644);
@@ -426,15 +447,17 @@ bool fgInitHome()
         if( ::unlink(ps.c_str()) != 0 ) // delete file when app quits
           result = false;
     }
-#endif
-
-    fgSetBool("/sim/fghome-readonly", false);
 
     if (!result) {
-        flightgear::fatalMessageBox("File permissions problem",
-                                    "Can't write to user-data storage folder, check file permissions and FG_HOME.",
-                                    "User-data at:" + dataPath.utf8Str());
+        flightgear::fatalMessageBoxWithoutExit(
+            "File permissions problem",
+            "Can't write to user-data storage folder, check file permissions "
+            "and FG_HOME.",
+            "User-data at '" + dataPath.utf8Str() + "'.");
+        return false;
     }
+#endif
+    fgSetBool("/sim/fghome-readonly", false);
     return result;
 }
 
@@ -463,10 +486,33 @@ int fgInitConfig ( int argc, char **argv, bool reinit )
         options->init(argc, argv, dataPath);
     }
 
-    // Read global preferences from $FG_ROOT/preferences.xml
-    SG_LOG(SG_GENERAL, SG_INFO, "Reading global preferences");
-    fgLoadProps("preferences.xml", globals->get_props());
-    SG_LOG(SG_GENERAL, SG_INFO, "Finished Reading global preferences");
+    // establish default for developer-mode based upon compiled build types
+    bool developerMode = true;
+    if (!strcmp(FG_BUILD_TYPE, "Release")) {
+        developerMode = false;
+    }
+
+    // allow command line to override
+    if (options->isOptionSet("developer")) {
+        string s = options->valueForOption("developer", "yes");
+        developerMode = simgear::strutils::to_bool(s);
+    }
+
+    fgSetBool("/sim/developer-mode", developerMode);
+    sglog().setDeveloperMode(developerMode);
+
+    // Read global defaults from $FG_ROOT/defaults
+    SG_LOG(SG_GENERAL, SG_INFO, "Reading global defaults");
+    SGPath defaultsXML = globals->get_fg_root() / "defaults.xml";
+    if (!defaultsXML.exists()) {
+        flightgear::fatalMessageBoxThenExit(
+            "Missing file",
+            "Couldn't load an essential simulator data file.",
+            defaultsXML.utf8Str());
+    }
+
+    fgLoadProps("defaults.xml", globals->get_props());
+    SG_LOG(SG_GENERAL, SG_INFO, "Finished Reading global defaults");
 
     // do not load user settings when reset to default is requested, or if
     // told to explicitly ignore
@@ -549,9 +595,10 @@ int fgInitAircraft(bool reinit)
         } else {
 #if 0
             // naturally the better option would be to on-demand install it!
-            flightgear::fatalMessageBox("Aircraft not installed",
-                                        "Requested aircraft is not currently installed.",
-                                        aircraftId);
+            flightgear::fatalMessageBoxWithoutExit(
+                "Aircraft not installed",
+                "Requested aircraft is not currently installed.",
+                aircraftId);
 
             return flightgear::FG_OPTIONS_ERROR;
 #endif
@@ -616,7 +663,7 @@ fgInitNav ()
   
   SGPath path(globals->get_fg_root());
   path.append( "Navaids/TACAN_freq.dat" );
-  flightgear::loadTacan(path, channellist);
+  flightgear::NavLoader().loadTacan(path, channellist);
   
   return true;
 }
@@ -735,6 +782,7 @@ void fgCreateSubsystems(bool duringReset) {
     if (!globals->get_subsystem<FGHTTPClient>()) {
         globals->add_new_subsystem<FGHTTPClient>();
     }
+    globals->add_new_subsystem<FGDNSClient>();
 
     ////////////////////////////////////////////////////////////////////
     // Initialize the flight model subsystem.
@@ -1012,7 +1060,7 @@ void fgStartNewReset()
         string_list::const_iterator it;
         for (it = names.begin(); it != names.end(); ++it) {
             if ((*it == "time") || (*it == "terrasync") || (*it == "events")
-                || (*it == "lighting") || (*it == FGTileMgr::subsystemName()) || (*it == FGScenery::subsystemName()))
+                || (*it == "lighting") || (*it == FGScenery::subsystemName()))
             {
                 continue;
             }
@@ -1035,7 +1083,6 @@ void fgStartNewReset()
     
     // order is important here since tile-manager shutdown needs to
     // access the scenery object
-    subsystemManger->remove(FGTileMgr::subsystemName());
     subsystemManger->remove(FGScenery::subsystemName());
 
     FGScenery::getPagerSingleton()->clearRequests();
@@ -1094,9 +1141,9 @@ void fgStartNewReset()
 
     viewer->getDatabasePager()->setUpThreads(1, 1);
     
-    // must do this before splashinit for Rembrandt
+    // must do this before preinit for Rembrandt
     flightgear::CameraGroup::buildDefaultGroup(viewer.get());
-    render->splashinit();
+    render->preinit();
     viewer->startThreading();
     
     fgOSResetProperties();
