@@ -20,25 +20,25 @@
 //
 // $Id$
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
 #include <boost/foreach.hpp>
 #include <algorithm>
 
+#ifndef FG_TESTLIB
 #include <osgViewer/Viewer>
 #include <osgDB/Registry>
+#endif
 
 #include <simgear/structure/commands.hxx>
+#include <simgear/structure/exception.hxx>
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/misc/sg_dir.hxx>
 #include <simgear/timing/sg_time.hxx>
 #include <simgear/ephemeris/ephemeris.hxx>
-#include <simgear/scene/material/matlib.hxx>
 #include <simgear/structure/subsystem_mgr.hxx>
 #include <simgear/structure/event_mgr.hxx>
-#include <simgear/sound/soundmgr.hxx>
+
 #include <simgear/misc/ResourceManager.hxx>
 #include <simgear/props/propertyObject.hxx>
 #include <simgear/props/props_io.hxx>
@@ -48,12 +48,20 @@
 #include <Aircraft/controls.hxx>
 #include <Airports/runways.hxx>
 #include <Autopilot/route_mgr.hxx>
+#include <Navaids/navlist.hxx>
+
 #include <GUI/gui.h>
+#include <Viewer/viewmgr.hxx>
+
+#ifndef FG_TESTLIB
 #include <Scenery/scenery.hxx>
 #include <Scenery/tilemgr.hxx>
-#include <Navaids/navlist.hxx>
 #include <Viewer/renderer.hxx>
-#include <Viewer/viewmgr.hxx>
+#include <GUI/FGFontCache.hxx>
+
+#include <simgear/sound/soundmgr.hxx>
+#include <simgear/scene/material/matlib.hxx>
+#endif
 
 #include "globals.hxx"
 #include "locale.hxx"
@@ -80,16 +88,6 @@ public:
     const char* aircraftDir = fgGetString("/sim/aircraft-dir");
     string_list aircraftDirPieces(sgPathBranchSplit(aircraftDir));
     if (!aircraftDirPieces.empty() && (aircraftDirPieces.back() == pieces[1])) {
-        // disable this warning for release builds
-#if defined(ENABLE_DEV_WARNINGS)
-        SGPath betterPath;
-        for (unsigned int i=2; i<pieces.size(); ++i) {
-            betterPath.append(pieces[i]);
-        }
-        SG_LOG(SG_AIRCRAFT, SG_WARN, "resolved path:" << aResource << " using /sim/aircraft-dir.\n"
-               "This is legacy behaviour, change to an aircraft relative path:" << betterPath);
-#endif
-
         // current aircraft-dir matches resource aircraft
         SGPath r(aircraftDir);
         for (unsigned int i=2; i<pieces.size(); ++i) {
@@ -148,7 +146,9 @@ FGGlobals *globals = NULL;
 
 // Constructor
 FGGlobals::FGGlobals() :
+#ifndef FG_TESTLIB
     renderer( new FGRenderer ),
+#endif
     subsystem_mgr( new SGSubsystemMgr ),
     event_mgr( new SGEventMgr ),
     sim_time_sec( 0.0 ),
@@ -196,7 +196,7 @@ FGGlobals::~FGGlobals()
 
     // stop OSG threading first, to avoid thread races while we tear down
     // scene-graph pieces
-
+#ifndef FG_TESTLIB
     osg::ref_ptr<osgViewer::Viewer> vw(renderer->getViewer());
     if (vw) {
         // https://code.google.com/p/flightgear-bugs/issues/detail?id=1291
@@ -205,11 +205,11 @@ FGGlobals::~FGGlobals()
         // GraphicsContext)
         vw->stopThreading();
     }
-
+#endif
     subsystem_mgr->shutdown();
     subsystem_mgr->unbind();
 
-    subsystem_mgr->remove(FGTileMgr::subsystemName());
+#ifndef FG_TESTLIB
     // don't cancel the pager until after shutdown, since AIModels (and
     // potentially others) can queue delete requests on the pager.
     if (vw && vw->getDatabasePager()) {
@@ -223,20 +223,26 @@ FGGlobals::~FGGlobals()
     // renderer touches subsystems during its destruction
     set_renderer(NULL);
 
+    FGFontCache::shutdown();
+    fgCancelSnapShot();
+#endif
+
     delete subsystem_mgr;
     subsystem_mgr = NULL; // important so ::get_subsystem returns NULL
-    vw = 0; // don't delete the viewer until now
+#ifndef FG_TESTLIB
+    vw = nullptr; // don't delete the viewer until now
+    set_matlib(NULL);
+#endif
 
     delete time_params;
-    set_matlib(NULL);
-
     delete channel_options_list;
     delete initial_waypoints;
     delete channellist;
 
     simgear::PropertyObjectBase::setDefaultRoot(NULL);
+#ifndef FG_TESTLIB
     simgear::SGModelLib::resetPropertyRoot();
-
+#endif
     delete locale;
     locale = NULL;
 
@@ -331,7 +337,7 @@ SGPath FGGlobals::find_data_dir(const std::string& pathSuffix) const
 
 void FGGlobals::append_fg_scenery (const PathList &paths)
 {
-    BOOST_FOREACH(const SGPath& path, paths) {
+    for (const SGPath& path : paths) {
         append_fg_scenery(path);
     }
 }
@@ -363,37 +369,16 @@ void FGGlobals::append_fg_scenery (const SGPath &path)
     // needed to load Models from this scenery path
     simgear::ResourceManager::instance()->addBasePath(abspath.local8BitStr(),
                                                       simgear::ResourceManager::PRIORITY_DEFAULT);
-
     simgear::Dir dir(abspath);
-    SGPath terrainDir(dir.file("Terrain"));
-    SGPath objectsDir(dir.file("Objects"));
 
-    // this code used to add *either* the base dir, OR add the
-    // Terrain and Objects subdirs, but the conditional logic was commented
-    // out, such that all three dirs are added. Unfortunately there's
-    // no information as to why the change was made.
     fg_scenery.push_back(abspath);
-    unmangled_fg_scenery.push_back(abspath);
     extra_read_allowed_paths.push_back(abspath);
 
-    if (terrainDir.exists()) {
-        fg_scenery.push_back(terrainDir);
-    }
-
-    if (objectsDir.exists()) {
-        fg_scenery.push_back(objectsDir);
-    }
-
-    // insert a marker for FGTileEntry::load(), so that
-    // FG_SCENERY=A:B becomes list ["A/Terrain", "A/Objects", "",
-    // "B/Terrain", "B/Objects", ""]
-    fg_scenery.push_back(SGPath());
-
     // make scenery dirs available to Nasal
     SGPropertyNode* n = sim->getChild("fg-scenery", propIndex++, true);
     n->setStringValue(abspath.utf8Str());
     n->setAttribute(SGPropertyNode::WRITE, false);
-    
+
     // temporary fix so these values survive reset
     n->setAttribute(SGPropertyNode::PRESERVE, true);
 }
@@ -411,9 +396,7 @@ void FGGlobals::append_read_allowed_paths(const SGPath &path)
 void FGGlobals::clear_fg_scenery()
 {
   fg_scenery.clear();
-  unmangled_fg_scenery.clear();
   fgGetNode("/sim", true)->removeChildren("fg-scenery");
-
 }
 
 // The 'path' argument to this method must come from trustworthy code, because
@@ -531,12 +514,14 @@ FGGlobals::get_renderer () const
 
 void FGGlobals::set_renderer(FGRenderer *render)
 {
+  #ifndef FG_TESTLIB
     if (render == renderer) {
         return;
     }
 
     delete renderer;
     renderer = render;
+#endif
 }
 
 SGSubsystemMgr *
@@ -690,9 +675,106 @@ SGPath FGGlobals::autosaveFilePath(SGPath userDataPath) const
     return simgear::Dir(userDataPath).file(autosaveName());
 }
 
+static void deleteProperties(SGPropertyNode* props, const string_list& blacklist)
+{
+    const std::string path(props->getPath());
+    auto it = std::find_if(blacklist.begin(), blacklist.end(), [path](const std::string& black)
+                           { return simgear::strutils::matchPropPathToTemplate(path, black); });
+    if (it != blacklist.end()) {
+        SGPropertyNode* pr = props->getParent();
+        pr->removeChild(props);
+        return;
+    }
+
+    // recurse
+    for (int c=0; c < props->nChildren(); ++c) {
+        deleteProperties(props->getChild(c), blacklist);
+    }
+
+}
+
+using VersionPair = std::pair<int, int>;
+
+static bool operator<(const VersionPair& a, const VersionPair& b)
+{
+    if (a.first == b.first) {
+        return a.second < b.second;
+    }
+
+    return a.first < b.first;
+}
+
+static void tryAutosaveMigration(const SGPath& userDataPath, SGPropertyNode* props)
+{
+    const string_list versionParts = simgear::strutils::split(VERSION, ".");
+    if (versionParts.size() < 2) {
+        return;
+    }
+
+    simgear::Dir userDataDir(userDataPath);
+    SGPath migratePath;
+    VersionPair foundVersion(0, 0);
+    const VersionPair currentVersion(simgear::strutils::to_int(versionParts[0]),
+                                     simgear::strutils::to_int(versionParts[1]));
+
+    for (auto previousSave : userDataDir.children(simgear::Dir::TYPE_FILE, ".xml")) {
+        const std::string base = previousSave.file_base();
+        VersionPair v;
+        // extract version from name
+        const int matches = ::sscanf(base.c_str(), "autosave_%d_%d", &v.first, &v.second);
+        if (matches != 2) {
+            continue;
+        }
+        
+        if (currentVersion < v) {
+            // ignore autosaves from more recent versions; this happens when
+            // running unsable and stable at the same time
+            continue;
+        }
+
+        if (v.first < 2000) {
+            // ignore autosaves from older versions, too much change to deal
+            // with.
+            continue;
+        }
+
+        if (foundVersion < v) {
+            foundVersion = v;
+            migratePath = previousSave;
+        }
+    }
+
+    if (!migratePath.exists()) {
+        return;
+    }
+
+    SG_LOG(SG_GENERAL, SG_INFO, "Migrating old autosave:" << migratePath);
+    SGPropertyNode oldProps;
+
+    try {
+        readProperties(migratePath, &oldProps, SGPropertyNode::USERARCHIVE);
+    } catch (sg_exception& e) {
+        SG_LOG(SG_GENERAL, SG_WARN, "failed to read previous user settings:" << e.getMessage()
+               << "(from " << e.getOrigin() << ")");
+        return;
+    }
+
+    // read migration blacklist
+    string_list blacklist;
+    for (auto node : fgGetNode("/sim/autosave-migration/blacklist")->getChildren("path")) {
+        blacklist.push_back(node->getStringValue());
+    }
+
+    // apply migration filters for each property in turn
+    deleteProperties(&oldProps, blacklist);
+
+    // copy remaining props out
+    copyProperties(&oldProps, props);
+}
+
 // Load user settings from the autosave file (normally in $FG_HOME)
 void
-FGGlobals::loadUserSettings(SGPath userDataPath)
+FGGlobals::loadUserSettings(SGPath userDataPath, bool tryMigrate)
 {
     if (userDataPath.isNull()) {
         userDataPath = get_fg_home();
@@ -712,6 +794,8 @@ FGGlobals::loadUserSettings(SGPath userDataPath)
           SG_LOG(SG_INPUT, SG_WARN, "failed to read user settings:" << e.getMessage()
             << "(from " << e.getOrigin() << ")");
       }
+    } else if (tryMigrate) {
+        tryAutosaveMigration(userDataPath, &autosave);
     }
     copyProperties(&autosave, globals->get_props());
 }
@@ -772,12 +856,11 @@ void FGGlobals::set_warp_delta( long int d )
 
 FGScenery* FGGlobals::get_scenery () const
 {
+#ifdef FG_TESTLIB
+    return nullptr;
+#else
     return get_subsystem<FGScenery>();
-}
-
-FGTileMgr* FGGlobals::get_tile_mgr () const
-{
-    return get_subsystem<FGTileMgr>();
+#endif
 }
 
 FGViewMgr *FGGlobals::get_viewmgr() const
@@ -793,7 +876,9 @@ flightgear::View* FGGlobals::get_current_view () const
 
 void FGGlobals::set_matlib( SGMaterialLib *m )
 {
+#ifndef FG_TESTLIB
     matlib = m;
+#endif
 }
 
 FGControls *FGGlobals::get_controls() const
