@@ -55,11 +55,9 @@ extern bool global_crashRptEnabled;
 #include <simgear/math/SGMath.hxx>
 #include <simgear/math/sg_random.h>
 #include <simgear/misc/strutils.hxx>
-#include <simgear/scene/tgdb/GroundLightManager.hxx>
 
 #include <Model/panelnode.hxx>
 #include <Scenery/scenery.hxx>
-#include <Scenery/tilemgr.hxx>
 #include <Sound/soundmanager.hxx>
 #include <Time/TimeManager.hxx>
 #include <GUI/gui.h>
@@ -127,16 +125,16 @@ static void initTerrasync()
     // (even if we are in read-only mode)
     SGPath terraSyncDir(globals->get_terrasync_dir());
     globals->append_data_path(terraSyncDir);
-    
+
     if (fgGetBool("/sim/fghome-readonly", false)) {
         return;
     }
-    
+
     // start TerraSync up now, so it can be synchronizing shared models
     // and airports data in parallel with a nav-cache rebuild.
     SGPath tsyncCache(terraSyncDir);
     tsyncCache.append("terrasync-cache.xml");
-    
+
     // wipe the cache file if requested
     if (flightgear::Options::sharedInstance()->isOptionSet("restore-defaults")) {
         SG_LOG(SG_GENERAL, SG_INFO, "restore-defaults requested, wiping terrasync update cache at " <<
@@ -145,7 +143,7 @@ static void initTerrasync()
             tsyncCache.remove();
         }
     }
-    
+
     fgSetString("/sim/terrasync/cache-path", tsyncCache.utf8Str());
 
     // make fg-root dir available so existing Scenery data can be copied, and
@@ -155,7 +153,7 @@ static void initTerrasync()
     simgear::SGTerraSync* terra_sync = new simgear::SGTerraSync();
     terra_sync->setRoot(globals->get_props());
     globals->add_subsystem("terrasync", terra_sync);
-    
+
     terra_sync->bind();
     terra_sync->init();
 }
@@ -205,7 +203,7 @@ static void checkOpenGLVersion()
     // versions doesn't help them much
     return;
 #endif
-    
+
     // format of these strings is not standardised, so be careful about
     // parsing them.
     std::string versionString(fgGetString("/sim/rendering/gl-version"));
@@ -250,7 +248,7 @@ static void fgIdleFunction ( void ) {
     // Specify our current idle function state.  This is used to run all
     // our initializations out of the idle callback so that we can get a
     // splash screen up and running right away.
-    
+
     if ( idle_state == 0 ) {
         if (guiInit())
         {
@@ -267,7 +265,7 @@ static void fgIdleFunction ( void ) {
         fgSplashProgress("loading-nav-dat");
 
     } else if ( idle_state == 3 ) {
-        
+
         bool done = fgInitNav();
         if (done) {
           ++idle_state;
@@ -278,7 +276,7 @@ static void fgIdleFunction ( void ) {
 
         TimeManager* t = new TimeManager;
         globals->add_subsystem("time", t, SGSubsystemMgr::INIT);
-        
+
         // Do some quick general initializations
         if( !fgInitGeneral()) {
             throw sg_exception("General initialization failed");
@@ -288,6 +286,7 @@ static void fgIdleFunction ( void ) {
         // Initialize the property-based built-in commands
         ////////////////////////////////////////////////////////////////////
         fgInitCommands();
+				fgInitSceneCommands();
 
         flightgear::registerSubsystemCommands(globals->get_commands());
 
@@ -296,26 +295,25 @@ static void fgIdleFunction ( void ) {
         ////////////////////////////////////////////////////////////////////
         globals->set_matlib( new SGMaterialLib );
         simgear::SGModelLib::setPanelFunc(FGPanelNode::load);
- 
+
     } else if (( idle_state == 5 ) || (idle_state == 2005)) {
         idle_state+=2;
         flightgear::initPosition();
         flightgear::initTowerLocationListener();
-        
+
         simgear::SGModelLib::init(globals->get_fg_root().local8BitStr(), globals->get_props());
-        
+
         TimeManager* timeManager = (TimeManager*) globals->get_subsystem("time");
         timeManager->init();
-        
+
         ////////////////////////////////////////////////////////////////////
         // Initialize the TG scenery subsystem.
         ////////////////////////////////////////////////////////////////////
-        
+
         globals->add_new_subsystem<FGScenery>(SGSubsystemMgr::DISPLAY);
         globals->get_scenery()->init();
         globals->get_scenery()->bind();
-        globals->add_new_subsystem<FGTileMgr>(SGSubsystemMgr::DISPLAY);
-        
+
         fgSplashProgress("creating-subsystems");
     } else if (( idle_state == 7 ) || (idle_state == 2007)) {
         bool isReset = (idle_state == 2007);
@@ -325,7 +323,7 @@ static void fgIdleFunction ( void ) {
         fgCreateSubsystems(isReset);
         SG_LOG(SG_GENERAL, SG_INFO, "Creating subsystems took:" << st.elapsedMSec());
         fgSplashProgress("binding-subsystems");
-      
+
     } else if ( idle_state == 8 ) {
         idle_state++;
         SGTimeStamp st;
@@ -342,14 +340,14 @@ static void fgIdleFunction ( void ) {
         } else {
           fgSplashProgress("init-subsystems");
         }
-      
+
     } else if ( idle_state == 10 ) {
         idle_state = 900;
         fgPostInitSubsystems();
         fgSplashProgress("finalize-position");
     } else if ( idle_state == 900 ) {
         idle_state = 1000;
-        
+
         // setup OpenGL view parameters
         globals->get_renderer()->setupView();
 
@@ -365,12 +363,14 @@ static void fgIdleFunction ( void ) {
     }
 
     if ( idle_state == 1000 ) {
+        sglog().setStartupLoggingEnabled(false);
+
         // We've finished all our initialization steps, from now on we
         // run the main loop.
         fgSetBool("sim/sceneryloaded", false);
         registerMainLoop();
     }
-    
+
     if ( idle_state == 2000 ) {
         fgStartNewReset();
         idle_state = 2005;
@@ -403,6 +403,18 @@ static void upper_case_property(const char *name)
     p->addChangeListener(muc);
 }
 
+// this hack is needed to avoid weird viewport sizing within OSG on Windows.
+// still required as of March 2017, sad times.
+// see for example https://sourceforge.net/p/flightgear/codetickets/1958/
+static void ATIScreenSizeHack()
+{
+    osg::ref_ptr<osg::Camera> hackCam = new osg::Camera;
+    hackCam->setRenderOrder(osg::Camera::PRE_RENDER);
+    int prettyMuchAnyInt = 1;
+    hackCam->setViewport(0, 0, prettyMuchAnyInt, prettyMuchAnyInt);
+    globals->get_renderer()->addCamera(hackCam, false);
+}
+
 // Propose NVIDIA Optimus / AMD Xpress to use high-end GPU
 #if defined(SG_WINDOWS)
 extern "C" {
@@ -411,7 +423,7 @@ extern "C" {
 }
 #endif
 
-static void logToFile()
+static void logToHome()
 {
     SGPath logPath = globals->get_fg_home();
     logPath.append("fgfs.log");
@@ -438,22 +450,25 @@ static void logToFile()
 // Main top level initialization
 int fgMainInit( int argc, char **argv )
 {
-    // set default log levels
-    sglog().setLogLevels( SG_ALL, SG_ALERT );
+    // set default log level to 'info' for startup, we will revert to a lower
+    // level once startup is done.
+    sglog().setLogLevels( SG_ALL, SG_INFO );
+    sglog().setStartupLoggingEnabled(true);
 
     globals = new FGGlobals;
     if (!fgInitHome()) {
         return EXIT_FAILURE;
     }
-    
+
     if (!fgGetBool("/sim/fghome-readonly")) {
         // now home is initialised, we can log to a file inside it
-        logToFile();
+        logToHome();
     }
-    
+
     std::string version(FLIGHTGEAR_VERSION);
     SG_LOG( SG_GENERAL, SG_INFO, "FlightGear:  Version "
             << version );
+    SG_LOG( SG_GENERAL, SG_INFO, "FlightGear:  Build Type " << FG_BUILD_TYPE );
     SG_LOG( SG_GENERAL, SG_INFO, "Built with " << SG_COMPILER_STR);
 	SG_LOG( SG_GENERAL, SG_INFO, "Jenkins number/ID " << HUDSON_BUILD_NUMBER << ":"
 			<< HUDSON_BUILD_ID);
@@ -495,10 +510,16 @@ int fgMainInit( int argc, char **argv )
 #if defined(HAVE_QT)
     if (showLauncher) {
         flightgear::initApp(argc, argv);
+        flightgear::checkKeyboardModifiersForSettingFGRoot();
+
         if (!flightgear::runLauncherDialog()) {
             return EXIT_SUCCESS;
         }
     }
+#else
+    if (showLauncher) {
+        SG_LOG(SG_GENERAL, SG_ALERT, "\n!Launcher requested, but FlightGear was compiled without Qt support!");
+    }
 #endif
     fgInitAircraftPaths(false);
 
@@ -508,14 +529,14 @@ int fgMainInit( int argc, char **argv )
     } else if (configResult == flightgear::FG_OPTIONS_EXIT) {
         return EXIT_SUCCESS;
     }
-    
+
     configResult = flightgear::Options::sharedInstance()->processOptions();
     if (configResult == flightgear::FG_OPTIONS_ERROR) {
         return EXIT_FAILURE;
     } else if (configResult == flightgear::FG_OPTIONS_EXIT) {
         return EXIT_SUCCESS;
     }
-    
+
     // Initialize the Window/Graphics environment.
     fgOSInit(&argc, argv);
     _bootstrap_OSInit++;
@@ -528,34 +549,35 @@ int fgMainInit( int argc, char **argv )
     // Clouds3D requires an alpha channel
     fgOSOpenWindow(true /* request stencil buffer */);
     fgOSResetProperties();
-    
-    // Initialize the splash screen right away
+
     fntInit();
-    fgSplashInit();
+    globals->get_renderer()->preinit();
+
+    if (fgGetBool("/sim/ati-viewport-hack", true)) {
+        SG_LOG(SG_GENERAL, SG_WARN, "Enabling ATI/AMD viewport hack");
+        ATIScreenSizeHack();
+    }
 
     fgOutputSettings();
-    
+
     //try to disable the screensaver
     fgOSDisableScreensaver();
-    
+
     // pass control off to the master event handler
     int result = fgOSMainLoop();
     frame_signal.clear();
     fgOSCloseWindow();
-    
+
     simgear::clearEffectCache();
-    
+
     // clean up here; ensure we null globals to avoid
     // confusing the atexit() handler
     delete globals;
     globals = NULL;
-    
+
     // delete the NavCache here. This will cause the destruction of many cached
     // objects (eg, airports, navaids, runways).
     delete flightgear::NavDataCache::instance();
-    simgear::GroundLightManager::instance()->getRunwayLightStateSet()->clear();
-    simgear::GroundLightManager::instance()->getTaxiLightStateSet()->clear();
-    simgear::GroundLightManager::instance()->getGroundLightStateSet()->clear();
-  
+
     return result;
 }
