@@ -35,10 +35,11 @@
 #include <3rdparty/cjson/cJSON.h>
 
 #include <cmath>        // rint()
-#include <stdio.h>
-#include <stdlib.h>     // atof(), atoi()
-#include <string.h>     // strcmp()
+#include <cstdio>
+#include <cstdlib>      // atof(), atoi()
+#include <cstring>      // strcmp()
 #include <algorithm>
+#include <map>
 
 #include <iostream>
 #include <string>
@@ -46,8 +47,9 @@
 
 #include <simgear/math/sg_random.h>
 #include <simgear/props/props_io.hxx>
-#include <simgear/misc/sgstream.hxx>
+#include <simgear/io/iostreams/sgstream.hxx>
 #include <simgear/misc/sg_path.hxx>
+#include <simgear/misc/sg_dir.hxx>
 #include <simgear/scene/material/mat.hxx>
 #include <simgear/sound/soundmgr.hxx>
 #include <simgear/misc/strutils.hxx>
@@ -57,7 +59,8 @@
 
 #include <GUI/gui.h>
 #include <GUI/MessageBox.hxx>
-#if defined(HAVE_QT)
+
+#if defined(HAVE_QT) && !defined(FG_TESTLIB)
 #include <GUI/QtLauncher.hxx>
 #include <GUI/SetupRootDialog.hxx>
 #endif
@@ -93,7 +96,7 @@ using namespace flightgear;
 
 #define NEW_DEFAULT_MODEL_HZ 120
 
-static flightgear::Options* shared_instance = NULL;
+static flightgear::Options* shared_instance = nullptr;
 
 static double
 atof( const string& str )
@@ -112,24 +115,24 @@ static int fgSetupProxy( const char *arg );
 /**
  * Set a few fail-safe default property values.
  *
- * These should all be set in $FG_ROOT/preferences.xml, but just
+ * These should all be set in $FG_ROOT/defaults.xml, but just
  * in case, we provide some initial sane values here. This method
  * should be invoked *before* reading any init files.
  */
 void fgSetDefaults ()
 {
 
-				// Position (deliberately out of range)
+    // Position (deliberately out of range)
     fgSetDouble("/position/longitude-deg", 9999.0);
     fgSetDouble("/position/latitude-deg", 9999.0);
     fgSetDouble("/position/altitude-ft", -9999.0);
 
-				// Orientation
+    // Orientation
     fgSetDouble("/orientation/heading-deg", 9999.0);
     fgSetDouble("/orientation/roll-deg", 0.0);
     fgSetDouble("/orientation/pitch-deg", 0.424);
 
-				// Velocities
+    // Velocities
     fgSetDouble("/velocities/uBody-fps", 0.0);
     fgSetDouble("/velocities/vBody-fps", 0.0);
     fgSetDouble("/velocities/wBody-fps", 0.0);
@@ -139,7 +142,7 @@ void fgSetDefaults ()
     fgSetDouble("/velocities/airspeed-kt", 0.0);
     fgSetDouble("/velocities/mach", 0.0);
 
-                                // Presets
+    // Presets
     fgSetDouble("/sim/presets/longitude-deg", 9999.0);
     fgSetDouble("/sim/presets/latitude-deg", 9999.0);
     fgSetDouble("/sim/presets/altitude-ft", -9999.0);
@@ -161,7 +164,7 @@ void fgSetDefaults ()
     fgSetBool("/sim/presets/onground", true);
     fgSetBool("/sim/presets/trim", false);
 
-				// Miscellaneous
+    // Miscellaneous
     fgSetBool("/sim/startup/splash-screen", true);
     // we want mouse-pointer to have an undefined value if nothing is
     // specified so we can do the right thing for voodoo-1/2 cards.
@@ -169,7 +172,7 @@ void fgSetDefaults ()
     fgSetBool("/controls/flight/auto-coordination", false);
     fgSetString("/sim/logging/priority", "alert");
 
-				// Features
+    // Features
     fgSetBool("/sim/hud/color/antialiased", false);
     fgSetBool("/sim/hud/enable3d[1]", true);
     fgSetBool("/sim/hud/visibility[1]", false);
@@ -178,13 +181,21 @@ void fgSetDefaults ()
     fgSetBool("/sim/sound/working", true);
     fgSetBool("/sim/fgcom/enabled", false);
 
-				// Flight Model options
+    // Flight Model options
     fgSetString("/sim/flight-model", "jsb");
     fgSetString("/sim/aero", "c172");
     fgSetInt("/sim/model-hz", NEW_DEFAULT_MODEL_HZ);
     fgSetDouble("/sim/speed-up", 1.0);
 
-				// Rendering options
+    // Scenery
+    fgSetString("/sim/scenery/engine", "tilecache");
+
+    // ( scenery = pagedLOD )
+    fgSetString("/sim/scenery/lod-levels", "1 3 5 7 9");
+    fgSetString("/sim/scenery/lod-res", "1");
+    fgSetString("/sim/scenery/lod-texturing", "bluemarble");
+
+    // Rendering options
     fgSetString("/sim/rendering/fog", "nicest");
     fgSetBool("/environment/clouds/status", true);
     fgSetBool("/sim/startup/fullscreen", false);
@@ -203,16 +214,16 @@ void fgSetDefaults ()
     fgSetString("/sim/view-mode", "pilot");
     fgSetDouble("/sim/current-view/heading-offset-deg", 0);
 
-				// HUD options
+    // HUD options
     fgSetString("/sim/startup/units", "feet");
     fgSetString("/sim/hud/frame-stat-type", "tris");
 
-				// Time options
+    // Time options
     fgSetInt("/sim/startup/time-offset", 0);
     fgSetString("/sim/startup/time-offset-type", "system-offset");
     fgSetLong("/sim/time/cur-time-override", 0);
 
-                                // Freeze options
+    // Freeze options
     fgSetBool("/sim/freeze/master", false);
     fgSetBool("/sim/freeze/position", false);
     fgSetBool("/sim/freeze/clock", false);
@@ -227,18 +238,18 @@ void fgSetDefaults ()
     SGPropertyNode* v = globals->get_props()->getNode("/sim/version", true);
     v->setValueReadOnly("flightgear", FLIGHTGEAR_VERSION);
     v->setValueReadOnly("simgear", SG_STRINGIZE(SIMGEAR_VERSION));
+#ifndef FG_TESTLIB
     v->setValueReadOnly("openscenegraph", osgGetVersion());
+#if OSG_VERSION_LESS_THAN(3,5,2)
     v->setValueReadOnly("openscenegraph-thread-safe-reference-counting",
                          osg::Referenced::getThreadSafeReferenceCounting());
+#endif
+#endif
     v->setValueReadOnly("revision", REVISION);
     v->setValueReadOnly("build-number", HUDSON_BUILD_NUMBER);
     v->setValueReadOnly("build-id", HUDSON_BUILD_ID);
     v->setValueReadOnly("hla-support", bool(FG_HAVE_HLA));
-#if defined(FG_NIGHTLY)
-    v->setValueReadOnly("nightly-build", true);
-#else
-    v->setValueReadOnly("nightly-build", false);
-#endif
+    v->setValueReadOnly("build-type", FG_BUILD_TYPE);
 
     char* envp = ::getenv( "http_proxy" );
     if( envp != NULL )
@@ -322,8 +333,7 @@ private:
 
     int getNumMaturity(const char * str)
     {
-        // changes should also be reflected in $FG_ROOT/data/options.xml &
-        // $FG_ROOT/data/Translations/string-default.xml
+        // Changes should also be reflected in $FG_ROOT/options.xml
         const char* levels[] = {"alpha","beta","early-production","production"};
 
         if (!strcmp(str, "all")) {
@@ -985,12 +995,14 @@ fgOptJpgHttpd( const char * arg )
 static int
 fgOptHttpd( const char * arg )
 {
+#ifndef FG_TESTLIB
     // port may be any valid address:port notation
     // like 127.0.0.1:8080
     // or just the port 8080
     string port = simgear::strutils::strip(string(arg));
     if( port.empty() ) return FG_OPTIONS_ERROR;
     fgSetString( string(flightgear::http::PROPERTY_ROOT).append("/options/listening-port").c_str(), port );
+#endif
     return FG_OPTIONS_OK;
 }
 
@@ -1054,6 +1066,50 @@ fgOptLogClasses( const char *arg )
     return FG_OPTIONS_OK;
 }
 
+static int
+fgOptLogDir(const char* arg)
+{
+    SGPath dirPath;
+    if (!strcmp(arg, "desktop")) {
+        dirPath = SGPath::desktop();
+    } else {
+        dirPath = SGPath::fromLocal8Bit(arg);
+    }
+
+    if (!dirPath.isDir()) {
+        SG_LOG(SG_GENERAL, SG_ALERT, "cannot find logging location " << dirPath);
+        return FG_OPTIONS_ERROR;
+    }
+
+    if (!dirPath.canWrite()) {
+        SG_LOG(SG_GENERAL, SG_ALERT, "cannot write to logging location " << dirPath);
+        return FG_OPTIONS_ERROR;
+    }
+
+    // generate the log file name
+    SGPath logFile;
+    {
+        char fileNameBuffer[100];
+        time_t now;
+        time(&now);
+        strftime(fileNameBuffer, 99, "FlightGear_%F", localtime(&now));
+
+        unsigned int logsTodayCount = 0;
+        while (true) {
+            std::ostringstream os;
+            os << fileNameBuffer << "_" << logsTodayCount++ << ".log";
+            logFile = dirPath / os.str();
+            if (!logFile.exists()) {
+                break;
+            }
+        }
+    }
+
+    sglog().logToFile(logFile, sglog().get_log_classes(), sglog().get_log_priority());
+
+    return FG_OPTIONS_OK;
+}
+
 static int
 fgOptTraceWrite( const char *arg )
 {
@@ -1304,6 +1360,14 @@ fgOptScenario( const char *arg )
     return FG_OPTIONS_OK;
 }
 
+static int
+fgOptAirport( const char *arg )
+{
+    fgSetString("/sim/presets/airport-id", arg );
+    fgSetBool("/sim/presets/airport-requested", true );
+    return FG_OPTIONS_OK;
+}
+
 static int
 fgOptRunway( const char *arg )
 {
@@ -1536,7 +1600,7 @@ struct OptionDesc {
     {"disable-sound",                false, OPTION_BOOL,   "/sim/sound/working", false, "", 0 },
     {"enable-sound",                 false, OPTION_BOOL,   "/sim/sound/working", true, "", 0 },
     {"sound-device",                 true,  OPTION_STRING, "/sim/sound/device-name", false, "", 0 },
-    {"airport",                      true,  OPTION_STRING, "/sim/presets/airport-id", false, "", 0 },
+    {"airport",                      true,  OPTION_FUNC,   "", false, "", fgOptAirport },
     {"runway",                       true,  OPTION_FUNC,   "", false, "", fgOptRunway },
     {"vor",                          true,  OPTION_FUNC,   "", false, "", fgOptVOR },
     {"vor-frequency",                true,  OPTION_DOUBLE, "/sim/presets/vor-freq", false, "", fgOptVOR },
@@ -1565,6 +1629,11 @@ struct OptionDesc {
     {"roc",                          true,  OPTION_FUNC,   "", false, "", fgOptRoc },
     {"fg-root",                      true,  OPTION_IGNORE,   "", false, "", 0 },
     {"fg-scenery",                   true,  OPTION_FUNC | OPTION_MULTI,   "", false, "", fgOptFgScenery },
+    {"terrain-engine",               true,  OPTION_STRING, "/sim/scenery/engine", false, "tilecache", 0 },
+    {"lod-levels",                   true,  OPTION_STRING, "/sim/scenery/lod-levels", false, "1 3 5 7", 0 },
+    {"lod-res",                      true,  OPTION_STRING, "/sim/scenery/lod-res", false, "1", 0 },
+    {"lod-texturing",                true,  OPTION_STRING, "/sim/scenery/lod-texturing", false, "bluemarble", 0 },
+    {"lod-range-mult",               true,  OPTION_STRING, "/sim/scenery/lod-range-mult", false, "2", 0 },
     {"fg-aircraft",                  true,  OPTION_IGNORE | OPTION_MULTI,   "", false, "", 0 },
     {"fdm",                          true,  OPTION_STRING, "/sim/flight-model", false, "", 0 },
     {"aero",                         true,  OPTION_STRING, "/sim/aero", false, "", 0 },
@@ -1661,6 +1730,7 @@ struct OptionDesc {
     {"trace-write",                  true,  OPTION_FUNC | OPTION_MULTI,   "", false, "", fgOptTraceWrite },
     {"log-level",                    true,  OPTION_FUNC,   "", false, "", fgOptLogLevel },
     {"log-class",                    true,  OPTION_FUNC,   "", false, "", fgOptLogClasses },
+    {"log-dir",                      true,  OPTION_FUNC | OPTION_MULTI, "", false, "", fgOptLogDir },
     {"view-offset",                  true,  OPTION_FUNC | OPTION_MULTI,   "", false, "", fgOptViewOffset },
     {"visibility",                   true,  OPTION_FUNC,   "", false, "", fgOptVisibilityMeters },
     {"visibility-miles",             true,  OPTION_FUNC,   "", false, "", fgOptVisibilityMiles },
@@ -1697,6 +1767,7 @@ struct OptionDesc {
     {"no-default-config",            false, OPTION_IGNORE, "", false, "", 0},
     {"prop",                         true,  OPTION_FUNC | OPTION_MULTI,   "", false, "", fgOptSetProperty},
     {"load-tape",                    true,  OPTION_FUNC,   "", false, "", fgOptLoadTape },
+    {"developer",                    true,  OPTION_IGNORE | OPTION_BOOL, "", false, "", nullptr },
     {0}
 };
 
@@ -1871,11 +1942,11 @@ public:
   cJSON *createJSONArrayFromPathList(const PathList& pl) const
   {
     cJSON *resultNode = cJSON_CreateArray();
+    cJSON *prevNode = nullptr;
     bool isFirst = true;
 
     for (const SGPath& path : pl) {
       cJSON *pathNode = cJSON_CreateString(path.utf8Str().c_str());
-      cJSON *prevNode;
 
       if (isFirst) {
         isFirst = false;
@@ -1911,6 +1982,14 @@ Options* Options::sharedInstance()
   return shared_instance;
 }
 
+void Options::reset()
+{
+    if (shared_instance != nullptr) {
+        delete shared_instance;
+        shared_instance = nullptr;
+    }
+}
+
 Options::Options() :
   p(new OptionsPrivate())
 {
@@ -1942,7 +2021,7 @@ void Options::init(int argc, char **argv, const SGPath& appDataPath)
         continue;
       }
 
-      int result = parseOption(argv[i]);
+      int result = parseOption(argv[i], /* fromConfigFile */ false);
       processArgResult(result);
     } else {
     // XML properties file
@@ -1966,7 +2045,9 @@ void Options::init(int argc, char **argv, const SGPath& appDataPath)
   }
 
   if (!p->shouldLoadDefaultConfig) {
+#if !defined(FG_TESTLIB)
     setupRoot(argc, argv);
+#endif
     return;
   }
 
@@ -1999,25 +2080,31 @@ void Options::init(int argc, char **argv, const SGPath& appDataPath)
 
 // system.fgfsrc is disabled, as we no longer allow anything in fgdata to set
 // fg-root/fg-home/fg-aircraft and hence control what files Nasal can access
-  std::string name_for_error = config.utf8Str();
+  std::string nameForError = config.utf8Str();
   if( ! hostname.empty() ) {
     config = globals->get_fg_root();
     config.append( "system.fgfsrc" );
     config.concat( "." );
     config.concat( hostname );
     if (config.exists()) {
-      flightgear::fatalMessageBox("Unsupported configuration",
-        "You have a " + config.utf8Str() + " file, which is no longer processed for security reasons",
-        "If you created this file intentionally, please move it to " + name_for_error);
+      flightgear::fatalMessageBoxThenExit(
+        "Unsupported configuration",
+        "You have a '" + config.utf8Str() + "' file, which is no longer "
+        "processed for security reasons.",
+        "If you created this file intentionally, please move it to '" +
+        nameForError + "'.");
     }
   }
 
   config = globals->get_fg_root();
   config.append( "system.fgfsrc" );
   if (config.exists()) {
-    flightgear::fatalMessageBox("Unsupported configuration",
-      "You have a " + config.utf8Str() + " file, which is no longer processed for security reasons",
-      "If you created this file intentionally, please move it to " + name_for_error);
+    flightgear::fatalMessageBoxThenExit(
+      "Unsupported configuration",
+      "You have a '" + config.utf8Str() + "' file, which is no longer "
+      "processed for security reasons.",
+      "If you created this file intentionally, please move it to '" +
+      nameForError + "'.");
   }
 }
 
@@ -2145,7 +2232,7 @@ void Options::readConfig(const SGPath& path)
         break;
     line = line.substr( 0, i );
 
-    if ( parseOption( line ) == FG_OPTIONS_ERROR ) {
+    if ( parseOption(line, /* fromConfigFile */ true) == FG_OPTIONS_ERROR ) {
       cerr << endl << "Config file parse error: " << path << " '"
       << line << "'" << endl;
 	    p->showHelp = true;
@@ -2156,7 +2243,7 @@ void Options::readConfig(const SGPath& path)
   p->insertGroupMarker(); // each config file is a group
 }
 
-int Options::parseOption(const string& s)
+int Options::parseOption(const string& s, bool fromConfigFile)
 {
   if ((s == "--help") || (s=="-h")) {
     return FG_OPTIONS_HELP;
@@ -2190,7 +2277,15 @@ int Options::parseOption(const string& s)
     SGPath path = s.substr(9);
     if (path.extension() == "xml") {
         p->propertyFiles.push_back(path);
-    } else {
+    } else if (fromConfigFile) {
+      flightgear::fatalMessageBoxThenExit(
+        "FlightGear",
+        "Invalid use of the --config option.",
+        "Sorry, it is currently not supported to load a configuration file "
+        "using --config from another configuration file.\n\n"
+        "Note: this does not apply to loading of XML PropertyList files "
+        "with --config.");
+    } else {                // the --config option comes from the command line
         p->configFiles.push_back(path);
     }
 
@@ -2466,7 +2561,7 @@ void Options::showUsage() const
   SGPropertyNode *options = options_root.getNode("options");
   if (!options) {
     SG_LOG( SG_GENERAL, SG_ALERT,
-           "Error reading options.xml: <options> directive not found." );
+           "Error reading options.xml: <options> element not found." );
     exit(-1);
   }
 
@@ -2476,8 +2571,8 @@ void Options::showUsage() const
       exit(-1);
   }
 
-  const char* usage = locale->getLocalizedString(options->getStringValue("usage"), "options");
-  if (usage) {
+  std::string usage = locale->getLocalizedString(options->getStringValue("usage"), "options");
+  if (!usage.empty()) {
     cout << usage << endl;
   }
 
@@ -2555,8 +2650,8 @@ void Options::showUsage() const
       }
     }
 
-    const char* name = locale->getLocalizedString(section[j]->getStringValue("name"),"options");
-    if (!msg.empty() && name) {
+    std::string name = locale->getLocalizedString(section[j]->getStringValue("name"),"options");
+    if (!msg.empty() && !name.empty()) {
       cout << endl << name << ":" << endl;
       cout << msg;
       msg.erase();
@@ -2564,8 +2659,8 @@ void Options::showUsage() const
   }
 
   if ( !p->verbose ) {
-    const char* verbose_help = locale->getLocalizedString(options->getStringValue("verbose-help"),"options");
-    if (verbose_help)
+    std::string verbose_help = locale->getLocalizedString(options->getStringValue("verbose-help"),"options");
+    if (!verbose_help.empty())
         cout << endl << verbose_help << endl;
   }
 #ifdef _MSC_VER
@@ -2579,14 +2674,17 @@ void Options::showVersion() const
     cout << "FlightGear version: " << FLIGHTGEAR_VERSION << endl;
     cout << "Revision: " << REVISION << endl;
     cout << "Build-Id: " << HUDSON_BUILD_ID << endl;
+    cout << "Build-Type: " << FG_BUILD_TYPE << endl;
     cout << "FG_ROOT=" << globals->get_fg_root().utf8Str() << endl;
     cout << "FG_HOME=" << globals->get_fg_home().utf8Str() << endl;
     cout << "FG_SCENERY=";
 
-    PathList scn = globals->get_unmangled_fg_scenery();
-    cout << SGPath::join(scn, &SGPath::pathListSep) << endl;
+    PathList scn = globals->get_fg_scenery();
+    cout << SGPath::join(scn, SGPath::pathListSep) << endl;
     cout << "SimGear version: " << SG_STRINGIZE(SIMGEAR_VERSION) << endl;
+#ifndef FG_TESTLIB
     cout << "OSG version: " << osgGetVersion() << endl;
+#endif
     cout << "PLIB version: " << PLIB_VERSION << endl;
 }
 
@@ -2615,6 +2713,7 @@ void Options::printJSONReport() const
   cJSON_AddStringToObject(generalNode, "name", "FlightGear");
   cJSON_AddStringToObject(generalNode, "version", FLIGHTGEAR_VERSION);
   cJSON_AddStringToObject(generalNode, "build ID", HUDSON_BUILD_ID);
+  cJSON_AddStringToObject(generalNode, "build type", FG_BUILD_TYPE);
 
   cJSON *configNode = cJSON_CreateObject();
   cJSON_AddItemToObject(rootNode, "config", configNode);
@@ -2623,8 +2722,7 @@ void Options::printJSONReport() const
   cJSON_AddStringToObject(configNode, "FG_HOME",
                           globals->get_fg_home().utf8Str().c_str());
 
-  cJSON *sceneryPathsNode = p->createJSONArrayFromPathList(
-    globals->get_unmangled_fg_scenery());
+  cJSON *sceneryPathsNode = p->createJSONArrayFromPathList(globals->get_fg_scenery());
   cJSON_AddItemToObject(configNode, "scenery paths", sceneryPathsNode);
 
   cJSON *aircraftPathsNode = p->createJSONArrayFromPathList(
@@ -2640,22 +2738,27 @@ void Options::printJSONReport() const
   cJSON_AddStringToObject(configNode, "autosave file",
                           globals->autosaveFilePath().utf8Str().c_str());
 
-  // Get the ordered list of apt.dat files used by the NavCache
+  // Get the ordered lists of apt.dat, fix.dat and nav.dat files used by the
+  // NavCache
   NavDataCache* cache = NavDataCache::instance();
   if (!cache) {
     cache = NavDataCache::createInstance();
   }
 
-  // For this method, it doesn't matter if the cache is out-of-date
-  NavDataCache::DatFilesGroupInfo aptDatFilesInfo =
-    cache->getDatFilesInfo(NavDataCache::DATFILETYPE_APT);
-
-  // Write the list to the JSON tree
   cJSON *navDataNode = cJSON_CreateObject();
   cJSON_AddItemToObject(rootNode, "navigation data", navDataNode);
-  cJSON *aptDatPathsNode = p->createJSONArrayFromPathList(
-    aptDatFilesInfo.paths);
-  cJSON_AddItemToObject(navDataNode, "apt.dat files", aptDatPathsNode);
+
+  // Write each list to the JSON tree
+  for (const auto& datType: {NavDataCache::DATFILETYPE_APT,
+                             NavDataCache::DATFILETYPE_FIX,
+                             NavDataCache::DATFILETYPE_NAV}) {
+    // For this method, it doesn't matter if the cache is out-of-date
+    const NavDataCache::DatFilesGroupInfo& datFilesInfo =
+      cache->getDatFilesInfo(datType);
+    cJSON *datPathsNode = p->createJSONArrayFromPathList(datFilesInfo.paths);
+    string key = NavDataCache::datTypeStr[datType] + ".dat files";
+    cJSON_AddItemToObject(navDataNode, key.c_str(), datPathsNode);
+  }
 
   // Print the JSON tree to the standard output
   char *report = cJSON_Print(rootNode);
@@ -2679,10 +2782,31 @@ SGPath Options::platformDefaultRoot() const
 #else
 SGPath Options::platformDefaultRoot() const
 {
-  return SGPath::fromUtf8(PKGLIBDIR);
+    return SGPath::fromUtf8(PKGLIBDIR);
 }
+
 #endif
 
+string_list Options::extractOptions() const
+{
+    string_list result;
+    for (auto opt : p->values) {
+        if (opt.desc == nullptr) {
+            continue;
+        }
+
+        if (!strcmp(opt.desc->option,"prop")) {
+            result.push_back("prop:" + opt.value);
+        } else if (opt.value.empty()) {
+            result.push_back(opt.desc->option);
+        } else {
+            result.push_back(std::string(opt.desc->option) + "=" + opt.value);
+        }
+    }
+
+    return result;
+}
+
 void Options::setupRoot(int argc, char **argv)
 {
     SGPath root;
@@ -2698,7 +2822,7 @@ void Options::setupRoot(int argc, char **argv)
         root = SGPath::fromLocal8Bit(envp);
         SG_LOG(SG_GENERAL, SG_INFO, "set from FG_ROOT env var: fg_root = " << root );
     } else {
-#if defined(HAVE_QT)
+#if defined(HAVE_QT) && !defined(FG_TESTLIB)
         flightgear::initApp(argc, argv);
         root = SetupRootDialog::restoreUserSelectedRoot();
 #endif
@@ -2713,36 +2837,41 @@ void Options::setupRoot(int argc, char **argv)
   }
 
   globals->set_fg_root(root);
-    static char required_version[] = FLIGHTGEAR_VERSION;
     string base_version = fgBasePackageVersion(root);
 
-#if defined(HAVE_QT)
+
+#if defined(HAVE_QT) && !defined(FG_TESTLIB)
+    // only compare major and minor version, not the patch level.
+    const int versionComp = simgear::strutils::compare_versions(FLIGHTGEAR_VERSION, base_version, 2);
+
     // note we never end up here if restoring a user selected root via
     // the Qt GUI, since that code pre-validates the path. But if we're using
     // a command-line, env-var or default root this check can fail and
     // we still want to use the GUI in that case
-    if (base_version != required_version) {
+    if (versionComp != 0) {
         flightgear::initApp(argc, argv);
         SetupRootDialog::runDialog(usingDefaultRoot);
     }
 #else
+    SG_UNUSED(usingDefaultRoot);
+
     // validate it
     if (base_version.empty()) {
-        flightgear::fatalMessageBox("Base package not found",
-                                    "Required data files not found, check your installation.",
-                                    "Looking for base-package files at: '" + root.str() + "'");
-
-        exit(-1);
+        flightgear::fatalMessageBoxThenExit(
+          "Base package not found",
+          "Required data files not found, please check your installation.",
+          "Looking for base-package files at: '" + root.str() + "'");
     }
 
-    if (base_version != required_version) {
-      flightgear::fatalMessageBox("Base package version mismatch",
-                                  "Version check failed: please check your installation.",
-                                  "Found data files for version '" + base_version +
-                                  "' at '" + globals->get_fg_root().str() + "', version '"
-                                  + required_version + "' is required.");
-
-    exit(-1);
+    // only compare major and minor version, not the patch level.
+    const int versionComp = simgear::strutils::compare_versions(FLIGHTGEAR_VERSION, base_version, 2);
+    if (versionComp != 0) {
+      flightgear::fatalMessageBoxThenExit(
+        "Base package version mismatch",
+        "Version check failed, please check your installation.",
+        "Found data files for version '" + base_version + "' at '" +
+        globals->get_fg_root().str() + "', version '" +
+        std::string(FLIGHTGEAR_VERSION) + "' is required.");
   }
 #endif
 }
