@@ -39,7 +39,7 @@
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/magvar/magvar.hxx>
 #include <simgear/timing/sg_time.hxx>
-#include <simgear/misc/sgstream.hxx>
+#include <simgear/io/iostreams/sgstream.hxx>
 #include <simgear/misc/strutils.hxx>
 #include <simgear/props/props_io.hxx>
 #include <simgear/xml/easyxml.hxx>
@@ -225,6 +225,12 @@ void FlightPlan::deleteIndex(int aIndex)
   
 void FlightPlan::clear()
 {
+    // some badly behaved CDU implementations call clear on a Nasal timer
+    // during startup.
+    if (_legs.empty() && (_currentIndex < 0)) {
+        return;
+    }
+
   lockDelegate();
   _waypointsChanged = true;
   _currentWaypointChanged = true;
@@ -1063,8 +1069,8 @@ WayptRef FlightPlan::waypointFromString(const string& tgt )
 
   SGGeod basePosition;
   if (_legs.empty()) {
-    // route is empty, use current position
-    basePosition = globals->get_aircraft_position();
+    // route is empty, use departure position / aircraft position
+    basePosition = _departure ? _departure->geod() : globals->get_aircraft_position();
   } else {
     basePosition = _legs.back()->waypoint()->position();
   }
