@@ -78,6 +78,7 @@
 #include <Airports/parking.hxx>
 #include <Airports/gnnode.hxx>
 #include "CacheSchema.h"
+#include <GUI/MessageBox.hxx>
 
 using std::string;
 
@@ -132,7 +133,7 @@ static string cleanRunwayNo(const string& aRwyNo)
   }
 
   string result(aRwyNo);
-  // canonicalise runway ident
+  // canonicalise runway ident to two digits and one optional uppercase letter
   if ((aRwyNo.size() == 1) || !isdigit(aRwyNo[1])) {
     result = "0" + aRwyNo;
   }
@@ -257,16 +258,32 @@ public:
 
   void init()
   {
-    SG_LOG(SG_NAVCACHE, SG_INFO, "NavCache at:" << path);
+      SG_LOG(SG_NAVCACHE, SG_INFO, "NavCache at:" << path);
 
       readOnly = fgGetBool("/sim/fghome-readonly", false);
+      SG_LOG(SG_NAVCACHE, SG_INFO, "NavCache read-only flags is:" << readOnly);
+
+      if (!readOnly && !path.canWrite()) {
+          throw sg_exception("Nav-cache file is not writeable");
+      }
 
       int openFlags = readOnly ? SQLITE_OPEN_READONLY :
         SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE;
-      // see http://code.google.com/p/flightgear-bugs/issues/detail?id=1055
-      // for the UTF8 / path logic here
       std::string pathUtf8 = path.utf8Str();
-    sqlite3_open_v2(pathUtf8.c_str(), &db, openFlags, NULL);
+      int result = sqlite3_open_v2(pathUtf8.c_str(), &db, openFlags, NULL);
+      if (result == SQLITE_MISUSE) {
+          // docs state sqlite3_errmsg may not work in this case
+          SG_LOG(SG_NAVCACHE, SG_WARN, "Failed to open DB at " << path << ": misuse of Sqlite API");
+          throw sg_exception("Navcache failed to open: Sqlite API misuse");
+      } else if (result != SQLITE_OK) {
+          std::string errMsg = sqlite3_errmsg(db);
+          SG_LOG(SG_NAVCACHE, SG_WARN, "Failed to open DB at " << path << " with error:\n\t" << errMsg);
+          throw sg_exception("Navcache failed to open:" + errMsg);
+      }
+
+      if (!readOnly && (sqlite3_db_readonly(db, nullptr) != 0)) {
+          throw sg_exception("Nav-cache file opened but is not writeable");
+      }
 
     sqlite3_stmt_ptr checkTables =
       prepare("SELECT count(*) FROM sqlite_master WHERE name='properties'");
@@ -334,10 +351,9 @@ public:
   }
 
   bool isCachedFileModified(const SGPath& path, bool verbose);
-  NavDataCache::DatFilesGroupInfo findDatFiles(
-    NavDataCache::DatFileType datFileType) const;
+  void findDatFiles(NavDataCache::DatFileType datFileType);
   bool areDatFilesModified(
-    const NavDataCache::DatFilesGroupInfo& datFilesGroupInfo,
+    NavDataCache::DatFileType datFileType,
     bool verbose);
 
   void callSqlite(int result, const string& sql)
@@ -855,8 +871,8 @@ public:
   bool transactionAborted;
   sqlite3_stmt_ptr beginTransactionStmt, commitTransactionStmt, rollbackTransactionStmt;
 
-  NavDataCache::DatFilesGroupInfo aptDatFilesInfo;
-  SGPath metarDatPath, navDatPath, fixDatPath, poiDatPath,
+  std::map<DatFileType, NavDataCache::DatFilesGroupInfo> datFilesInfo;
+  SGPath metarDatPath, poiDatPath,
   carrierDatPath, airwayDatPath;
 
   sqlite3_stmt_ptr readPropertyQuery, writePropertyQuery,
@@ -1022,14 +1038,15 @@ bool NavDataCache::NavDataCachePrivate::isCachedFileModified(const SGPath& path,
 // $FG_ROOT/Airports/apt.dat.gz for the 'apt' type).
 // Also compute the total size of all these files (in bytes), which is useful
 // for progress information.
-NavDataCache::DatFilesGroupInfo NavDataCache::NavDataCachePrivate::findDatFiles(
-  NavDataCache::DatFileType datFileType) const
+// The result is stored in the datFilesInfo field.
+void NavDataCache::NavDataCachePrivate::findDatFiles(
+  NavDataCache::DatFileType datFileType)
 {
   NavDataCache::DatFilesGroupInfo result;
   SGPath visitedPath;           // to avoid duplicates and time wasting
   const string datFilesSubDir = "NavData/" +
                                NavDataCache::datTypeStr[datFileType];
-  const PathList& sceneryPaths = globals->get_unmangled_fg_scenery();
+  const PathList& sceneryPaths = globals->get_fg_scenery();
   result.datFileType = datFileType;
   result.totalSize = 0;
 
@@ -1076,7 +1093,7 @@ NavDataCache::DatFilesGroupInfo NavDataCache::NavDataCachePrivate::findDatFiles(
     result.totalSize += defaultDatFile.sizeInBytes();
   }
 
-  return result;
+  datFilesInfo[datFileType] = result;
 }
 
 // Compare:
@@ -1088,14 +1105,16 @@ NavDataCache::DatFilesGroupInfo NavDataCache::NavDataCachePrivate::findDatFiles(
 // This comparison is sensitive to the number and order of the files,
 // their respective SGPath::realpath() and SGPath::modTime().
 bool NavDataCache::NavDataCachePrivate::areDatFilesModified(
-  const NavDataCache::DatFilesGroupInfo& datFilesGroupInfo,
+  NavDataCache::DatFileType datFileType,
   bool verbose)
 {
   // 'apt' or 'metar' or 'fix' or...
+  const NavDataCache::DatFilesGroupInfo& datFilesGroupInfo =
+        datFilesInfo[datFileType];
   const string datTypeStr =
-                       NavDataCache::datTypeStr[datFilesGroupInfo.datFileType];
+                       NavDataCache::datTypeStr[datFileType];
   const string_list cachedFiles = outer->readOrderedStringListProperty(
-    datTypeStr + ".dat files", &SGPath::pathListSep);
+    datTypeStr + ".dat files", SGPath::pathListSep);
   const PathList& datFiles = datFilesGroupInfo.paths;
   PathList::const_iterator datFilesIt = datFiles.begin();
   string_list::const_iterator cachedFilesIt = cachedFiles.begin();
@@ -1181,48 +1200,64 @@ const string NavDataCache::defaultDatFile[] = {
 
 NavDataCache::NavDataCache()
 {
-  const int MAX_TRIES = 3;
-  SGPath homePath(globals->get_fg_home());
+    const int MAX_TRIES = 3;
+    SGPath homePath(globals->get_fg_home());
 
-  std::ostringstream os;
-  string_list versionParts = simgear::strutils::split(VERSION, ".");
-  if (versionParts.size() < 2) {
-    os << "navdata.cache";
-  } else {
-    os << "navdata_" << versionParts[0] << "_" << versionParts[1] << ".cache";
-  }
+    std::ostringstream os;
+    string_list versionParts = simgear::strutils::split(VERSION, ".");
+    if (versionParts.size() < 2) {
+        os << "navdata.cache";
+    } else {
+        os << "navdata_" << versionParts[0] << "_" << versionParts[1] << ".cache";
+    }
 
-  homePath.append(os.str());
+    homePath.append(os.str());
 
     // permit additional DB connections from the same process
     sqlite3_config(SQLITE_CONFIG_MULTITHREAD);
 
-  for (int t=0; t < MAX_TRIES; ++t) {
-    try {
-      d.reset(new NavDataCachePrivate(homePath, this));
-      d->init();
-      //d->checkCacheFile();
-    // reached this point with no exception, success
-      break;
-    } catch (sg_exception& e) {
-      SG_LOG(SG_NAVCACHE, t == 0 ? SG_WARN : SG_ALERT, "NavCache: init failed:" << e.what()
-             << " (attempt " << t << ")");
-      d.reset();
+    for (int t=0; t < MAX_TRIES; ++t) {
+        try {
+            d.reset(new NavDataCachePrivate(homePath, this));
+            d->init();
+            //d->checkCacheFile();
+            // reached this point with no exception, success
+            break;
+        } catch (sg_exception& e) {
+            SG_LOG(SG_NAVCACHE, t == 0 ? SG_WARN : SG_ALERT, "NavCache: init failed:" << e.what()
+                   << " (attempt " << t << ")");
 
-        // only wipe the existing if not readonly
-        if (!fgGetBool("/sim/fghome-readonly", false)) {
-            homePath.remove();
+            if (t == (MAX_TRIES - 1)) {
+                // final attempt still failed, we are busted
+                flightgear::fatalMessageBoxThenExit(
+                  "Unable to open navigation cache",
+                  std::string("The navigation data cache could not be opened:")
+                  + e.getMessage(), e.getOrigin());
+            }
+
+            d.reset();
+
+            // only wipe the existing if not readonly
+            if (!fgGetBool("/sim/fghome-readonly", false)) {
+                bool ok = homePath.remove();
+                if (!ok) {
+                    SG_LOG(SG_NAVCACHE, SG_ALERT, "NavCache: failed to remove previous cache file");
+                    flightgear::fatalMessageBoxThenExit(
+                      "Unable to re-create navigation cache",
+                      "Attempting to remove the old cache failed.",
+                      "Location: "  + homePath.utf8Str());
+                }
+            }
         }
-    }
-  } // of retry loop
-
-  double RADIUS_EARTH_M = 7000 * 1000.0; // 7000km is plenty
-  SGVec3d earthExtent(RADIUS_EARTH_M, RADIUS_EARTH_M, RADIUS_EARTH_M);
-  Octree::global_spatialOctree =
+    } // of retry loop
+    
+    double RADIUS_EARTH_M = 7000 * 1000.0; // 7000km is plenty
+    SGVec3d earthExtent(RADIUS_EARTH_M, RADIUS_EARTH_M, RADIUS_EARTH_M);
+    Octree::global_spatialOctree =
     new Octree::Branch(SGBox<double>(-earthExtent, earthExtent), 1);
-
-  // Update d->aptDatFilesInfo, d->metarDatPath, d->navDatPath, etc.
-  updateListsOfDatFiles();
+    
+    // Update d->aptDatFilesInfo, d->metarDatPath, d->navDatPath, etc.
+    updateListsOfDatFiles();
 }
 
 NavDataCache::~NavDataCache()
@@ -1248,23 +1283,18 @@ NavDataCache* NavDataCache::instance()
 void NavDataCache::updateListsOfDatFiles() {
   // All $scenery_path/NavData/apt/*.dat[.gz] files found inside scenery
   // paths, plus $FG_ROOT/Airports/apt.dat.gz (order matters).
-  d->aptDatFilesInfo = d->findDatFiles(DATFILETYPE_APT);
-  // Trivial extension to the other types of dat files:
-  // d->navDatFilesInfo = d->findDatFiles(DATFILETYPE_NAV);
-  // d->fixDatFilesInfo = d->findDatFiles(DATFILETYPE_FIX);
-  // etc.
-  //
+  d->findDatFiles(DATFILETYPE_APT);
+
+  // All $scenery_path/NavData/{nav,fix}/*.dat[.gz] files found inside scenery
+  // paths, plus $FG_ROOT/Navaids/{nav,fix}.dat.gz (order matters).
+  d->findDatFiles(DATFILETYPE_NAV);
+  d->findDatFiles(DATFILETYPE_FIX);
+
   // These ones are still managed the "old" way (no search through scenery
   // paths).
   d->metarDatPath = SGPath(globals->get_fg_root());
   d->metarDatPath.append("Airports/metar.dat.gz");
 
-  d->navDatPath = SGPath(globals->get_fg_root());
-  d->navDatPath.append("Navaids/nav.dat.gz");
-
-  d->fixDatPath = SGPath(globals->get_fg_root());
-  d->fixDatPath.append("Navaids/fix.dat.gz");
-
   d->poiDatPath = SGPath(globals->get_fg_root());
   d->poiDatPath.append("Navaids/poi.dat.gz");
 
@@ -1275,20 +1305,16 @@ void NavDataCache::updateListsOfDatFiles() {
   d->airwayDatPath.append("Navaids/awy.dat.gz");
 }
 
-NavDataCache::DatFilesGroupInfo
+const NavDataCache::DatFilesGroupInfo&
 NavDataCache::getDatFilesInfo(DatFileType datFileType) const
 {
-  switch (datFileType) {
-    case DATFILETYPE_APT:
-      return d->aptDatFilesInfo;
-    default:
-      SG_LOG(SG_NAVCACHE, SG_ALERT,
-             "NavCache: requesting info about the list of " <<
-             datTypeStr[datFileType] << " dat files, however this is not "
-             "implemented yet!");
-      assert(false);
-      return DatFilesGroupInfo();
+  auto iter = d->datFilesInfo.find(datFileType);
+  if (iter == d->datFilesInfo.end()) {
+    throw sg_error("NavCache: requesting info about the list of " +
+           datTypeStr[datFileType] + " dat files, however this is not "
+           "implemented yet!");
   }
+  return iter->second;
 }
 
 bool NavDataCache::isRebuildRequired()
@@ -1302,10 +1328,10 @@ bool NavDataCache::isRebuildRequired()
         return true;
     }
 
-  if (d->areDatFilesModified(d->aptDatFilesInfo, true) ||
+  if (d->areDatFilesModified(DATFILETYPE_APT, true) ||
       d->isCachedFileModified(d->metarDatPath, true) ||
-      d->isCachedFileModified(d->navDatPath, true) ||
-      d->isCachedFileModified(d->fixDatPath, true) ||
+      d->areDatFilesModified(DATFILETYPE_NAV, true) ||
+      d->areDatFilesModified(DATFILETYPE_FIX, true) ||
       d->isCachedFileModified(d->carrierDatPath, true) ||
 // since POI loading is disabled on Windows, don't check for it
 // this caused: https://code.google.com/p/flightgear-bugs/issues/detail?id=1227
@@ -1355,8 +1381,41 @@ void NavDataCache::setRebuildPhaseProgress(RebuildPhase ph, unsigned int percent
     d->rebuilder->setProgress(ph, percent);
 }
 
+void NavDataCache::loadDatFiles(
+    DatFileType type,
+    std::function<void(const SGPath&, std::size_t, std::size_t)> loader)
+{
+  SGTimeStamp st;
+  string typeStr = datTypeStr[type];
+  string_list datFiles;
+  NavDataCache::DatFilesGroupInfo datFilesInfo = getDatFilesInfo(type);
+  const PathList datPaths = datFilesInfo.paths;
+  std::size_t bytesReadSoFar = 0;
+
+  st.stamp();
+  for (PathList::const_iterator it = datPaths.begin();
+       it != datPaths.end(); it++) {
+    string path = it->realpath().utf8Str();
+    datFiles.push_back(path);
+    SG_LOG(SG_GENERAL, SG_INFO,
+           "Loading " + typeStr + ".dat file: '" << path << "'");
+    loader(*it, bytesReadSoFar, datFilesInfo.totalSize);
+    bytesReadSoFar += it->sizeInBytes();
+    stampCacheFile(*it); // this uses the realpath() of the file
+  }
+
+  // Store the list of .dat files we have loaded
+  writeOrderedStringListProperty(typeStr + ".dat files", datFiles,
+                                 SGPath::pathListSep);
+  SG_LOG(SG_NAVCACHE, SG_INFO,
+         typeStr + ".dat files load took: " <<
+         st.elapsedMSec());
+}
+
 void NavDataCache::doRebuild()
 {
+  rebuildInProgress = true;
+
   try {
     d->close(); // completely close the sqlite object
     d->path.remove(); // remove the file on disk
@@ -1369,45 +1428,30 @@ void NavDataCache::doRebuild()
     {
         Transaction txn(this);
         APTLoader aptLoader;
-        string_list aptDatFiles;
-        const PathList aptDatPaths = d->aptDatFilesInfo.paths;
-        std::size_t bytesReadSoFar = 0;
+        FixesLoader fixesLoader;
+        NavLoader navLoader;
+
+        using namespace std::placeholders;  // for _1, _2, _3...
+
+        loadDatFiles(DATFILETYPE_APT,
+                     std::bind(&APTLoader::readAptDatFile, &aptLoader, _1, _2, _3));
 
         st.stamp();
-        for (PathList::const_iterator it = aptDatPaths.begin();
-             it != aptDatPaths.end(); it++) {
-            aptDatFiles.push_back(it->realpath().utf8Str());
-            aptLoader.readAptDatFile(*it, bytesReadSoFar,
-                                     d->aptDatFilesInfo.totalSize);
-            bytesReadSoFar += it->sizeInBytes();
-            stampCacheFile(*it); // this uses the realpath() of the file
-        }
-
         setRebuildPhaseProgress(REBUILD_UNKNOWN);
-        SG_LOG(SG_NAVCACHE, SG_DEBUG, "Loading airports");
-
+        SG_LOG(SG_NAVCACHE, SG_DEBUG, "Processing airports");
         aptLoader.loadAirports(); // load airport data into the NavCache
-        // Store the list of apt.dat files we have loaded
-        writeOrderedStringListProperty(
-          datTypeStr[DATFILETYPE_APT] + ".dat files", aptDatFiles,
-          &SGPath::pathListSep);
         SG_LOG(SG_NAVCACHE, SG_INFO,
-               datTypeStr[DATFILETYPE_APT] + ".dat files load took:" <<
+               "processing airports took:" <<
                st.elapsedMSec());
 
         setRebuildPhaseProgress(REBUILD_UNKNOWN);
         metarDataLoad(d->metarDatPath);
         stampCacheFile(d->metarDatPath);
 
-        st.stamp();
-        FixesLoader().loadFixes(d->fixDatPath);
-        stampCacheFile(d->fixDatPath);
-        SG_LOG(SG_NAVCACHE, SG_INFO, "fix.dat load took:" << st.elapsedMSec());
-
-        st.stamp();
-        navDBInit(d->navDatPath);
-        stampCacheFile(d->navDatPath);
-        SG_LOG(SG_NAVCACHE, SG_INFO, "nav.dat load took:" << st.elapsedMSec());
+        loadDatFiles(DATFILETYPE_FIX,
+                     std::bind(&FixesLoader::loadFixes, &fixesLoader, _1, _2, _3));
+        loadDatFiles(DATFILETYPE_NAV,
+                     std::bind(&NavLoader::loadNav, &navLoader, _1, _2, _3));
 
         setRebuildPhaseProgress(REBUILD_UNKNOWN);
         st.stamp();
@@ -1435,7 +1479,8 @@ void NavDataCache::doRebuild()
 
       {
           Transaction txn(this);
-          loadCarrierNav(d->carrierDatPath);
+          NavLoader navLoader;
+          navLoader.loadCarrierNav(d->carrierDatPath);
           stampCacheFile(d->carrierDatPath);
 
           st.stamp();
@@ -1457,6 +1502,8 @@ void NavDataCache::doRebuild()
   } catch (sg_exception& e) {
     SG_LOG(SG_NAVCACHE, SG_ALERT, "caught exception rebuilding navCache:" << e.what());
   }
+
+  rebuildInProgress = false;
 }
 
 int NavDataCache::readIntProperty(const string& key)
@@ -1669,7 +1716,7 @@ FGPositionedRef NavDataCache::loadById(PositionedID rowid)
   if (rowid == 0) {
     return NULL;
   }
-
+  if (!d) return NULL;
   PositionedCache::iterator it = d->cache.find(rowid);
   if (it != d->cache.end()) {
     d->cacheHits++;
@@ -1677,7 +1724,13 @@ FGPositionedRef NavDataCache::loadById(PositionedID rowid)
   }
 
   sqlite3_int64 aptId;
-  FGPositioned* pos = d->loadById(rowid, aptId);
+  FGPositionedRef pos = d->loadById(rowid, aptId);
+  if (rebuildInProgress) {
+    // Do not cache and apply ILS adjustment while rebuilding the cache.
+    // The adjustment process requires all ILS navaids to be present,
+    // which is not true during the cache rebuild.
+    return pos;
+  }
   d->cache.insert(it, PositionedCache::value_type(rowid, pos));
   d->cacheMisses++;
 
@@ -1904,7 +1957,7 @@ FGPositionedRef NavDataCache::findClosestWithIdent( const string& aIdent,
   FGPositionedRef result;
 
   while (d->stepSelect(d->findClosestWithIdent)) {
-    FGPositioned* pos = loadById(sqlite3_column_int64(d->findClosestWithIdent, 0));
+    FGPositionedRef pos = loadById(sqlite3_column_int64(d->findClosestWithIdent, 0));
     if (aFilter && !aFilter->pass(pos)) {
       continue;
     }
@@ -2044,7 +2097,7 @@ NavDataCache::findCommByFreq(int freqKhz, const SGGeod& aPos, FGPositioned::Filt
   FGPositionedRef result;
 
   while (d->execSelect(d->findCommByFreq)) {
-    FGPositioned* p = loadById(sqlite3_column_int64(d->findCommByFreq, 0));
+    FGPositionedRef p = loadById(sqlite3_column_int64(d->findCommByFreq, 0));
     if (aFilter && !aFilter->pass(p)) {
       continue;
     }
