@@ -85,7 +85,7 @@ FGPositioned::~FGPositioned()
 {
 }
 
-FGPositioned*
+FGPositionedRef
 FGPositioned::createUserWaypoint(const std::string& aIdent, const SGGeod& aPos)
 {
   NavDataCache* cache = NavDataCache::instance();
@@ -93,7 +93,7 @@ FGPositioned::createUserWaypoint(const std::string& aIdent, const SGGeod& aPos)
   FGPositionedList existing = cache->findAllWithIdent(aIdent, &filter, true);
   if (!existing.empty()) {
     SG_LOG(SG_NAVAID, SG_WARN, "attempt to insert duplicate WAYPOINT:" << aIdent);
-    return existing.front().ptr();
+    return existing.front();
   }
   
   PositionedID id = cache->createPOI(WAYPOINT, aIdent, aPos);
@@ -126,6 +126,8 @@ FGPositioned::Type FGPositioned::typeFromName(const std::string& aName)
   
   const NameTypeEntry names[] = {
     {"airport", AIRPORT},
+    {"heliport", HELIPORT},
+    {"seaport", SEAPORT},
     {"vor", VOR},
     {"loc", LOC},
     {"ils", ILS},
@@ -141,13 +143,24 @@ FGPositioned::Type FGPositioned::typeFromName(const std::string& aName)
     {"ground", FREQ_GROUND},
     {"approach", FREQ_APP_DEP},
     {"departure", FREQ_APP_DEP},
+    {"clearance", FREQ_CLEARANCE},
+    {"unicom", FREQ_UNICOM},
     {"runway", RUNWAY},
     {"helipad", HELIPAD},
     {"country", COUNTRY},
     {"city", CITY},
     {"town", TOWN},
     {"village", VILLAGE},
-      
+    {"taxiway", TAXIWAY},
+    {"pavement", PAVEMENT},
+    {"om", OM},
+    {"mm", MM},
+    {"im", IM},
+    {"mobile-tacan", MOBILE_TACAN},
+    {"obstacle", OBSTACLE},
+    {"parking", PARKING},
+    {"taxi-node",TAXI_NODE},
+
   // aliases
     {"localizer", LOC},
     {"gnd", FREQ_GROUND},
@@ -158,7 +171,11 @@ FGPositioned::Type FGPositioned::typeFromName(const std::string& aName)
     {"rwy", RUNWAY},
     {"any", INVALID},
     {"all", INVALID},
-    
+    {"outer-marker", OM},
+    {"middle-marker", MM},
+    {"inner-marker", IM},
+    {"parking-stand", PARKING},
+
     {NULL, INVALID}
   };
   
