@@ -59,7 +59,8 @@ void Listener::valueChanged(SGPropertyNode * node)
 }
 
 FGSoundManager::FGSoundManager()
-  : _is_initialized(false),
+  : _active_dt(0.0),
+    _is_initialized(false),
     _enabled(false),
     _listener(new Listener(this))
 {
@@ -163,7 +164,7 @@ bool FGSoundManager::stationaryView() const
 // Actual sound update is triggered by the subsystem manager.
 void FGSoundManager::update(double dt)
 {
-    if (_is_initialized && _sound_working->getBoolValue())
+    if (is_active() && _is_initialized && _sound_working->getBoolValue())
     {
         bool enabled = _sound_enabled->getBoolValue();
         if (enabled != _enabled)
@@ -189,7 +190,13 @@ void FGSoundManager::update(double dt)
 
             set_velocity( velocity );
 
-            set_volume(_volume->getFloatValue());
+            float vf = 1.0f;
+            if (_active_dt < 5.0) {
+                _active_dt += dt;
+                vf = std::min(std::pow(_active_dt*0.2, 5.0), 1.0);
+            }
+
+            set_volume(vf*_volume->getFloatValue());
             SGSoundMgr::update(dt);
         }
     }
