@@ -54,7 +54,7 @@
 
 #include <simgear/compiler.h>
 #include <simgear/misc/sg_path.hxx>
-#include <simgear/misc/sgstream.hxx>
+#include <simgear/io/iostreams/sgstream.hxx>
 #include <simgear/misc/sg_dir.hxx>
 #include <simgear/props/props.hxx>
 #include <simgear/structure/subsystem_mgr.hxx>
@@ -321,9 +321,7 @@ private:
 
         if (!FGAISchedule::validModelPath(mdl)) {
             missingModels.insert(mdl);
-#if defined(ENABLE_DEV_WARNINGS)
-            SG_LOG(SG_AI, SG_WARN, "TrafficMgr: Missing model path:" << mdl);
-#endif
+            SG_LOG(SG_AI, SG_DEV_WARN, "TrafficMgr: Missing model path:" << mdl);
             requiredAircraft = homePort = "";
             return;
         }
@@ -586,17 +584,13 @@ void FGTrafficManager::finishInit()
     assert(doingInit);
     SG_LOG(SG_AI, SG_INFO, "finishing AI-Traffic init");
     loadHeuristics();
-#if defined(ENABLE_DEV_WARNINGS)
     PerformanceDB* perfDB = globals->get_subsystem<PerformanceDB>();
-#endif
     // Do sorting and scoring separately, to take advantage of the "homeport" variable
     BOOST_FOREACH(FGAISchedule* schedule, scheduledAircraft) {
         schedule->setScore();
-#if defined(ENABLE_DEV_WARNINGS)
         if (!perfDB->havePerformanceDataForAircraftType(schedule->getAircraft())) {
-            SG_LOG(SG_AI, SG_WARN, "AI-Traffic: schedule aircraft missing performance data:" << schedule->getAircraft());
+            SG_LOG(SG_AI, SG_DEV_WARN, "AI-Traffic: schedule aircraft missing performance data:" << schedule->getAircraft());
         }
-#endif
     }
 
     sort(scheduledAircraft.begin(), scheduledAircraft.end(),
@@ -641,10 +635,8 @@ void FGTrafficManager::loadHeuristics()
               break;
             HeuristicMapIterator itr = heurMap.find(h.registration);
             if (itr != heurMap.end()) {
-#if defined(ENABLE_DEV_WARNINGS)
-              SG_LOG(SG_AI, SG_WARN,"Traffic Manager Warning: found duplicate tailnumber " <<
+              SG_LOG(SG_AI, SG_DEV_WARN,"Traffic Manager Warning: found duplicate tailnumber " <<
                      h.registration << " for AI aircraft");
-#endif
             } else {
               heurMap[h.registration] = h;
             }
