@@ -433,10 +433,8 @@ FGRenderer::~FGRenderer()
 }
 
 // Initialize various GL/view parameters
-// XXX This should be called "preinit" or something, as it initializes
-// critical parts of the scene graph in addition to the splash screen.
 void
-FGRenderer::splashinit( void )
+FGRenderer::preinit( void )
 {
     // important that we reset the viewer sceneData here, to ensure the reference
     // time for everything is in sync; otherwise on reset the Viewer and
@@ -446,10 +444,10 @@ FGRenderer::splashinit( void )
     _viewerSceneRoot = new osg::Group;
     _viewerSceneRoot->setName("viewerSceneRoot");
     viewer->setSceneData(_viewerSceneRoot);
-    
-    ref_ptr<Node> splashNode = fgCreateSplashNode();
+
+    _splash = new SplashScreen;
     if (_classicalRenderer) {
-        _viewerSceneRoot->addChild(splashNode.get());
+        _viewerSceneRoot->addChild(_splash);
     } else {
         for (   CameraGroup::CameraIterator ii = CameraGroup::getDefault()->camerasBegin();
                 ii != CameraGroup::getDefault()->camerasEnd();
@@ -459,7 +457,7 @@ FGRenderer::splashinit( void )
             Camera* camera = info->getCamera(DISPLAY_CAMERA);
             if (camera == 0) continue;
 
-            camera->addChild(splashNode.get());
+            camera->addChild(_splash);
         }
     }
     
@@ -470,6 +468,11 @@ FGRenderer::splashinit( void )
     _updateVisitor->setFrameStamp(_frameStamp.get());
     viewer->setUpdateVisitor(_updateVisitor.get());
     fgSetDouble("/sim/startup/splash-alpha", 1.0);
+    
+    // hide the menubar if it overlaps the window, so the splash screen
+    // is completely visible. We reset this value when the splash screen
+    // is fading out.
+    fgSetBool("/sim/menubar/overlap-hide", true);
 }
 
 class ShadowMapSizeListener : public SGPropertyChangeListener {
@@ -1596,6 +1599,7 @@ FGRenderer::update( ) {
         _splash_alpha->setDoubleValue((sAlpha < 0) ? 0.0 : sAlpha);
 
         syncPausePopupState();
+        fgSetBool("/sim/menubar/overlap-hide", false);
     }
 
     FGLight *l = static_cast<FGLight*>(globals->get_subsystem("lighting"));
@@ -1727,6 +1731,9 @@ FGRenderer::resize( int width, int height )
         _xsize->setIntValue(width);
         _ysize->setIntValue(height);
     }
+
+    // update splash node if present ?
+    _splash->resize(width, height);
 }
 
 typedef osgUtil::LineSegmentIntersector::Intersection Intersection;
